@@ -152,7 +152,10 @@ def create_cmd(
 
     asyncio.run(create_plots(Params(), plot_keys))
     if not exclude_final_dir:
-        add_plot_directory(root_path, final_dir)
+        try:
+            add_plot_directory(root_path, final_dir)
+        except ValueError as e:
+            print(e)
 
 
 @plots_cmd.command("check", short_help="Checks plots")
@@ -189,7 +192,11 @@ def check_cmd(
 def add_cmd(ctx: click.Context, final_dir: str):
     from chia.plotting.util import add_plot_directory
 
-    add_plot_directory(ctx.obj["root_path"], final_dir)
+    try:
+        add_plot_directory(ctx.obj["root_path"], final_dir)
+        print(f"Successfully added: {final_dir}")
+    except ValueError as e:
+        print(e)
 
 
 @plots_cmd.command("remove", short_help="Removes a directory of plots from config.yaml")
