@@ -1,15 +1,18 @@
+import asyncio
 import sys
 from pathlib import Path
+from typing import Any, Dict
 
 import click
 
+from chia.util.config import load_config
 from chia.util.service_groups import all_groups, services_for_groups
 
 
-async def async_stop(root_path: Path, group: str, stop_daemon: bool) -> int:
+async def async_stop(root_path: Path, config: Dict[str, Any], group: str, stop_daemon: bool) -> int:
     from chia.daemon.client import connect_to_daemon_and_validate
 
-    daemon = await connect_to_daemon_and_validate(root_path)
+    daemon = await connect_to_daemon_and_validate(root_path, config)
     if daemon is None:
         print("Couldn't connect to chia daemon")
         return 1
@@ -18,6 +21,9 @@ async def async_stop(root_path: Path, group: str, stop_daemon: bool) -> int:
         r = await daemon.exit()
         await daemon.close()
         if r.get("data", {}).get("success", False):
+            if r["data"].get("services_stopped") is not None:
+                [print(f"{service}: Stopped") for service in r["data"]["services_stopped"]]
+            await asyncio.sleep(1)  # just cosmetic
             print("Daemon stopped")
         else:
             print(f"Stop daemon failed {r}")
@@ -44,6 +50,6 @@ async def async_stop(root_path: Path, group: str, stop_daemon: bool) -> int:
 @click.argument("group", type=click.Choice(list(all_groups())), nargs=-1, required=True)
 @click.pass_context
 def stop_cmd(ctx: click.Context, daemon: bool, group: str) -> None:
-    import asyncio
-
-    sys.exit(asyncio.run(async_stop(ctx.obj["root_path"], group, daemon)))
+    root_path = ctx.obj["root_path"]
+    config = load_config(root_path, "config.yaml")
+    sys.exit(asyncio.run(async_stop(root_path, config, group, daemon)))
