@@ -211,10 +211,10 @@ async def validate_block_body(
         assert npc_result.conds is not None
 
         for spend in npc_result.conds.spends:
-            removals.append(spend.coin_id)
-            removals_puzzle_dic[spend.coin_id] = spend.puzzle_hash
+            removals.append(bytes32(spend.coin_id))
+            removals_puzzle_dic[bytes32(spend.coin_id)] = bytes32(spend.puzzle_hash)
             for puzzle_hash, amount, _ in spend.create_coin:
-                c = Coin(spend.coin_id, puzzle_hash, uint64(amount))
+                c = Coin(bytes32(spend.coin_id), bytes32(puzzle_hash), uint64(amount))
                 additions.append((c, c.name()))
     else:
         assert npc_result is None
@@ -316,7 +316,6 @@ async def validate_block_body(
                     min(constants.MAX_BLOCK_COST_CLVM, curr.transactions_info.cost),
                     cost_per_byte=constants.COST_PER_BYTE,
                     mempool_mode=False,
-                    height=curr.height,
                 )
                 removals_in_curr, additions_in_curr = tx_removals_and_additions(curr_npc_result.conds)
             else:
@@ -431,7 +430,7 @@ async def validate_block_body(
     assert_fee_sum: uint64 = uint64(0)
     if npc_result:
         assert npc_result.conds is not None
-        assert_fee_sum = npc_result.conds.reserve_fee
+        assert_fee_sum = uint64(npc_result.conds.reserve_fee)
 
     # 17. Check that the assert fee sum <= fees, and that each reserved fee is non-negative
     if fees < assert_fee_sum:
