@@ -1,18 +1,19 @@
 import logging
-from typing import Dict, List, Optional, Tuple, Any
+import sqlite3
+from typing import Dict, List, Optional, Tuple, Any, Union, Sequence
 
 import zstd
 
 from chia.consensus.block_record import BlockRecord
+from chia.types.blockchain_format.program import SerializedProgram
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.full_block import FullBlock
-from chia.types.blockchain_format.program import SerializedProgram
 from chia.types.weight_proof import SubEpochChallengeSegment, SubEpochSegments
-from chia.util.errors import Err
 from chia.util.db_wrapper import DBWrapper2
+from chia.util.errors import Err
+from chia.util.full_block_utils import generator_from_block
 from chia.util.ints import uint32
 from chia.util.lru_cache import LRUCache
-from chia.util.full_block_utils import generator_from_block
 
 log = logging.getLogger(__name__)
 
@@ -28,8 +29,9 @@ class BlockStore:
         # All full blocks which have been added to the blockchain. Header_hash -> block
         self.db_wrapper = db_wrapper
 
-        async with self.db_wrapper.write_db() as conn:
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
 
+            log.info("DB: Creating block store tables and indexes.")
             if self.db_wrapper.db_version == 2:
 
                 # TODO: most data in block is duplicated in block_record. The only
@@ -55,6 +57,7 @@ class BlockStore:
 
                 # If any of these indices are altered, they should also be altered
                 # in the chia/cmds/db_upgrade.py file
+                log.info("DB: Creating index height")
                 await conn.execute("CREATE INDEX IF NOT EXISTS height on full_blocks(height)")
 
                 # Sub epoch segments for weight proofs
@@ -66,10 +69,12 @@ class BlockStore:
 
                 # If any of these indices are altered, they should also be altered
                 # in the chia/cmds/db_upgrade.py file
+                log.info("DB: Creating index is_fully_compactified")
                 await conn.execute(
                     "CREATE INDEX IF NOT EXISTS is_fully_compactified ON"
                     " full_blocks(is_fully_compactified, in_main_chain) WHERE in_main_chain=1"
                 )
+                log.info("DB: Creating index main_chain")
                 await conn.execute(
                     "CREATE INDEX IF NOT EXISTS main_chain ON full_blocks(height, in_main_chain) WHERE in_main_chain=1"
                 )
@@ -95,13 +100,17 @@ class BlockStore:
                 )
 
                 # Height index so we can look up in order of height for sync purposes
+                log.info("DB: Creating index full_block_height")
                 await conn.execute("CREATE INDEX IF NOT EXISTS full_block_height on full_blocks(height)")
+                log.info("DB: Creating index is_fully_compactified")
                 await conn.execute(
                     "CREATE INDEX IF NOT EXISTS is_fully_compactified on full_blocks(is_fully_compactified)"
                 )
 
+                log.info("DB: Creating index height")
                 await conn.execute("CREATE INDEX IF NOT EXISTS height on block_records(height)")
 
+                log.info("DB: Creating index peak")
                 await conn.execute("CREATE INDEX IF NOT EXISTS peak on block_records(is_peak)")
 
         self.block_cache = LRUCache(1000)
@@ -129,16 +138,22 @@ class BlockStore:
         else:
             return FullBlock.from_bytes(block_bytes)
 
+    def maybe_decompress_blob(self, block_bytes: bytes) -> bytes:
+        if self.db_wrapper.db_version == 2:
+            return zstd.decompress(block_bytes)
+        else:
+            return block_bytes
+
     async def rollback(self, height: int) -> None:
         if self.db_wrapper.db_version == 2:
-            async with self.db_wrapper.write_db() as conn:
+            async with self.db_wrapper.writer_maybe_transaction() as conn:
                 await conn.execute(
                     "UPDATE OR FAIL full_blocks SET in_main_chain=0 WHERE height>? AND in_main_chain=1", (height,)
                 )
 
     async def set_in_chain(self, header_hashes: List[Tuple[bytes32]]) -> None:
         if self.db_wrapper.db_version == 2:
-            async with self.db_wrapper.write_db() as conn:
+            async with self.db_wrapper.writer_maybe_transaction() as conn:
                 await conn.executemany(
                     "UPDATE OR FAIL full_blocks SET in_main_chain=1 WHERE header_hash=?", header_hashes
                 )
@@ -155,7 +170,7 @@ class BlockStore:
 
         self.block_cache.put(header_hash, block)
 
-        async with self.db_wrapper.write_db() as conn:
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
             await conn.execute(
                 "UPDATE full_blocks SET block=?,is_fully_compactified=? WHERE header_hash=?",
                 (
@@ -176,7 +191,7 @@ class BlockStore:
                 else bytes(block_record.sub_epoch_summary_included)
             )
 
-            async with self.db_wrapper.write_db() as conn:
+            async with self.db_wrapper.writer_maybe_transaction() as conn:
                 await conn.execute(
                     "INSERT OR IGNORE INTO full_blocks VALUES(?, ?, ?, ?, ?, ?, ?, ?)",
                     (
@@ -192,7 +207,7 @@ class BlockStore:
                 )
 
         else:
-            async with self.db_wrapper.write_db() as conn:
+            async with self.db_wrapper.writer_maybe_transaction() as conn:
                 await conn.execute(
                     "INSERT OR IGNORE INTO full_blocks VALUES(?, ?, ?, ?, ?)",
                     (
@@ -222,7 +237,7 @@ class BlockStore:
     async def persist_sub_epoch_challenge_segments(
         self, ses_block_hash: bytes32, segments: List[SubEpochChallengeSegment]
     ) -> None:
-        async with self.db_wrapper.write_db() as conn:
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
             await conn.execute(
                 "INSERT OR REPLACE INTO sub_epoch_segments_v3 VALUES(?, ?)",
                 (self.maybe_to_hex(ses_block_hash), bytes(SubEpochSegments(segments))),
@@ -236,7 +251,7 @@ class BlockStore:
         if cached is not None:
             return cached
 
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             async with conn.execute(
                 "SELECT challenge_segments from sub_epoch_segments_v3 WHERE ses_block_hash=?",
                 (self.maybe_to_hex(ses_block_hash),),
@@ -263,7 +278,7 @@ class BlockStore:
             log.debug(f"cache hit for block {header_hash.hex()}")
             return cached
         log.debug(f"cache miss for block {header_hash.hex()}")
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             async with conn.execute(
                 "SELECT block from full_blocks WHERE header_hash=?", (self.maybe_to_hex(header_hash),)
             ) as cursor:
@@ -280,7 +295,7 @@ class BlockStore:
             log.debug(f"cache hit for block {header_hash.hex()}")
             return bytes(cached)
         log.debug(f"cache miss for block {header_hash.hex()}")
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             async with conn.execute(
                 "SELECT block from full_blocks WHERE header_hash=?", (self.maybe_to_hex(header_hash),)
             ) as cursor:
@@ -297,10 +312,9 @@ class BlockStore:
         if len(heights) == 0:
             return []
 
-        heights_db = tuple(heights)
-        formatted_str = f'SELECT block from full_blocks WHERE height in ({"?," * (len(heights_db) - 1)}?)'
-        async with self.db_wrapper.read_db() as conn:
-            async with conn.execute(formatted_str, heights_db) as cursor:
+        formatted_str = f'SELECT block from full_blocks WHERE height in ({"?," * (len(heights) - 1)}?)'
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            async with conn.execute(formatted_str, heights) as cursor:
                 ret: List[FullBlock] = []
                 for row in await cursor.fetchall():
                     ret.append(self.maybe_decompress(row[0]))
@@ -314,7 +328,7 @@ class BlockStore:
             return cached.transactions_generator
 
         formatted_str = "SELECT block, height from full_blocks WHERE header_hash=?"
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             async with conn.execute(formatted_str, (self.maybe_to_hex(header_hash),)) as cursor:
                 row = await cursor.fetchone()
                 if row is None:
@@ -341,13 +355,12 @@ class BlockStore:
             return []
 
         generators: Dict[uint32, SerializedProgram] = {}
-        heights_db = tuple(heights)
         formatted_str = (
             f"SELECT block, height from full_blocks "
-            f'WHERE in_main_chain=1 AND height in ({"?," * (len(heights_db) - 1)}?)'
+            f'WHERE in_main_chain=1 AND height in ({"?," * (len(heights) - 1)}?)'
         )
-        async with self.db_wrapper.read_db() as conn:
-            async with conn.execute(formatted_str, heights_db) as cursor:
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            async with conn.execute(formatted_str, heights) as cursor:
                 async for row in cursor:
                     block_bytes = zstd.decompress(row[0])
 
@@ -366,7 +379,7 @@ class BlockStore:
 
         return [generators[h] for h in heights]
 
-    async def get_block_records_by_hash(self, header_hashes: List[bytes32]):
+    async def get_block_records_by_hash(self, header_hashes: List[bytes32]) -> List[BlockRecord]:
         """
         Returns a list of Block Records, ordered by the same order in which header_hashes are passed in.
         Throws an exception if the blocks are not present
@@ -376,19 +389,19 @@ class BlockStore:
 
         all_blocks: Dict[bytes32, BlockRecord] = {}
         if self.db_wrapper.db_version == 2:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "SELECT header_hash,block_record FROM full_blocks "
                     f'WHERE header_hash in ({"?," * (len(header_hashes) - 1)}?)',
-                    tuple(header_hashes),
+                    header_hashes,
                 ) as cursor:
                     for row in await cursor.fetchall():
                         header_hash = bytes32(row[0])
                         all_blocks[header_hash] = BlockRecord.from_bytes(row[1])
         else:
             formatted_str = f'SELECT block from block_records WHERE header_hash in ({"?," * (len(header_hashes) - 1)}?)'
-            async with self.db_wrapper.read_db() as conn:
-                async with conn.execute(formatted_str, tuple([hh.hex() for hh in header_hashes])) as cursor:
+            async with self.db_wrapper.reader_no_transaction() as conn:
+                async with conn.execute(formatted_str, [hh.hex() for hh in header_hashes]) as cursor:
                     for row in await cursor.fetchall():
                         block_rec: BlockRecord = BlockRecord.from_bytes(row[0])
                         all_blocks[block_rec.header_hash] = block_rec
@@ -400,6 +413,41 @@ class BlockStore:
             ret.append(all_blocks[hh])
         return ret
 
+    async def get_block_bytes_by_hash(self, header_hashes: List[bytes32]) -> List[bytes]:
+        """
+        Returns a list of Full Blocks block blobs, ordered by the same order in which header_hashes are passed in.
+        Throws an exception if the blocks are not present
+        """
+
+        if len(header_hashes) == 0:
+            return []
+
+        # sqlite on python3.7 on windows has issues with large variable substitutions
+        assert len(header_hashes) < 901
+        header_hashes_db: Sequence[Union[bytes32, str]]
+        if self.db_wrapper.db_version == 2:
+            header_hashes_db = header_hashes
+        else:
+            header_hashes_db = [hh.hex() for hh in header_hashes]
+        formatted_str = (
+            f'SELECT header_hash, block from full_blocks WHERE header_hash in ({"?," * (len(header_hashes_db) - 1)}?)'
+        )
+        all_blocks: Dict[bytes32, bytes] = {}
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            async with conn.execute(formatted_str, header_hashes_db) as cursor:
+                for row in await cursor.fetchall():
+                    header_hash = bytes32(self.maybe_from_hex(row[0]))
+                    all_blocks[header_hash] = self.maybe_decompress_blob(row[1])
+
+        ret: List[bytes] = []
+        for hh in header_hashes:
+            block = all_blocks.get(hh)
+            if block is not None:
+                ret.append(block)
+            else:
+                raise ValueError(f"Header hash {hh} not in the blockchain")
+        return ret
+
     async def get_blocks_by_hash(self, header_hashes: List[bytes32]) -> List[FullBlock]:
         """
         Returns a list of Full Blocks blocks, ordered by the same order in which header_hashes are passed in.
@@ -409,16 +457,16 @@ class BlockStore:
         if len(header_hashes) == 0:
             return []
 
-        header_hashes_db: Tuple[Any, ...]
+        header_hashes_db: Sequence[Union[bytes32, str]]
         if self.db_wrapper.db_version == 2:
-            header_hashes_db = tuple(header_hashes)
+            header_hashes_db = header_hashes
         else:
-            header_hashes_db = tuple([hh.hex() for hh in header_hashes])
+            header_hashes_db = [hh.hex() for hh in header_hashes]
         formatted_str = (
             f'SELECT header_hash, block from full_blocks WHERE header_hash in ({"?," * (len(header_hashes_db) - 1)}?)'
         )
         all_blocks: Dict[bytes32, FullBlock] = {}
-        async with self.db_wrapper.read_db() as conn:
+        async with self.db_wrapper.reader_no_transaction() as conn:
             async with conn.execute(formatted_str, header_hashes_db) as cursor:
                 for row in await cursor.fetchall():
                     header_hash = bytes32(self.maybe_from_hex(row[0]))
@@ -436,7 +484,7 @@ class BlockStore:
 
         if self.db_wrapper.db_version == 2:
 
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "SELECT block_record FROM full_blocks WHERE header_hash=?",
                     (header_hash,),
@@ -446,7 +494,7 @@ class BlockStore:
                 return BlockRecord.from_bytes(row[0])
 
         else:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "SELECT block from block_records WHERE header_hash=?",
                     (header_hash.hex(),),
@@ -469,7 +517,7 @@ class BlockStore:
         ret: Dict[bytes32, BlockRecord] = {}
         if self.db_wrapper.db_version == 2:
 
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "SELECT header_hash, block_record FROM full_blocks WHERE height >= ? AND height <= ?",
                     (start, stop),
@@ -482,7 +530,7 @@ class BlockStore:
 
             formatted_str = f"SELECT header_hash, block from block_records WHERE height >= {start} and height <= {stop}"
 
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with await conn.execute(formatted_str) as cursor:
                     for row in await cursor.fetchall():
                         header_hash = bytes32(self.maybe_from_hex(row[0]))
@@ -490,22 +538,44 @@ class BlockStore:
 
         return ret
 
+    async def get_block_bytes_in_range(
+        self,
+        start: int,
+        stop: int,
+    ) -> List[bytes]:
+        """
+        Returns a list with all full blocks in range between start and stop
+        if present.
+        """
+
+        maybe_decompress_blob = self.maybe_decompress_blob
+        assert self.db_wrapper.db_version == 2
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            async with conn.execute(
+                "SELECT block FROM full_blocks WHERE height >= ? AND height <= ? and in_main_chain=1",
+                (start, stop),
+            ) as cursor:
+                rows: List[sqlite3.Row] = list(await cursor.fetchall())
+                if len(rows) != (stop - start) + 1:
+                    raise ValueError(f"Some blocks in range {start}-{stop} were not found.")
+                return [maybe_decompress_blob(row[0]) for row in rows]
+
     async def get_peak(self) -> Optional[Tuple[bytes32, uint32]]:
 
         if self.db_wrapper.db_version == 2:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute("SELECT hash FROM current_peak WHERE key = 0") as cursor:
                     peak_row = await cursor.fetchone()
             if peak_row is None:
                 return None
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute("SELECT height FROM full_blocks WHERE header_hash=?", (peak_row[0],)) as cursor:
                     peak_height = await cursor.fetchone()
             if peak_height is None:
                 return None
             return bytes32(peak_row[0]), uint32(peak_height[0])
         else:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute("SELECT header_hash, height from block_records WHERE is_peak = 1") as cursor:
                     peak_row = await cursor.fetchone()
             if peak_row is None:
@@ -527,7 +597,7 @@ class BlockStore:
         ret: Dict[bytes32, BlockRecord] = {}
         if self.db_wrapper.db_version == 2:
 
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "SELECT header_hash, block_record FROM full_blocks WHERE height >= ?",
                     (peak[1] - blocks_n,),
@@ -538,7 +608,7 @@ class BlockStore:
 
         else:
             formatted_str = f"SELECT header_hash, block  from block_records WHERE height >= {peak[1] - blocks_n}"
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(formatted_str) as cursor:
                     for row in await cursor.fetchall():
                         header_hash = bytes32(self.maybe_from_hex(row[0]))
@@ -552,10 +622,10 @@ class BlockStore:
 
         if self.db_wrapper.db_version == 2:
             # Note: we use the key field as 0 just to ensure all inserts replace the existing row
-            async with self.db_wrapper.write_db() as conn:
+            async with self.db_wrapper.writer_maybe_transaction() as conn:
                 await conn.execute("INSERT OR REPLACE INTO current_peak VALUES(?, ?)", (0, header_hash))
         else:
-            async with self.db_wrapper.write_db() as conn:
+            async with self.db_wrapper.writer_maybe_transaction() as conn:
                 await conn.execute("UPDATE block_records SET is_peak=0 WHERE is_peak=1")
                 await conn.execute(
                     "UPDATE block_records SET is_peak=1 WHERE header_hash=?",
@@ -563,7 +633,7 @@ class BlockStore:
                 )
 
     async def is_fully_compactified(self, header_hash: bytes32) -> Optional[bool]:
-        async with self.db_wrapper.write_db() as conn:
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
             async with conn.execute(
                 "SELECT is_fully_compactified from full_blocks WHERE header_hash=?", (self.maybe_to_hex(header_hash),)
             ) as cursor:
@@ -575,7 +645,7 @@ class BlockStore:
     async def get_random_not_compactified(self, number: int) -> List[int]:
 
         if self.db_wrapper.db_version == 2:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     f"SELECT height FROM full_blocks WHERE in_main_chain=1 AND is_fully_compactified=0 "
                     f"ORDER BY RANDOM() LIMIT {number}"
@@ -585,7 +655,7 @@ class BlockStore:
             # Since orphan blocks do not get compactified, we need to check whether all blocks with a
             # certain height are not compact. And if we do have compact orphan blocks, then all that
             # happens is that the occasional chain block stays uncompact - not ideal, but harmless.
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     f"SELECT height FROM full_blocks GROUP BY height HAVING sum(is_fully_compactified)=0 "
                     f"ORDER BY RANDOM() LIMIT {number}"
@@ -599,13 +669,13 @@ class BlockStore:
     async def count_compactified_blocks(self) -> int:
         if self.db_wrapper.db_version == 2:
             # DB V2 has an index on is_fully_compactified only for blocks in the main chain
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "select count(*) from full_blocks where is_fully_compactified=1 and in_main_chain=1"
                 ) as cursor:
                     row = await cursor.fetchone()
         else:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute("select count(*) from full_blocks where is_fully_compactified=1") as cursor:
                     row = await cursor.fetchone()
 
@@ -617,13 +687,13 @@ class BlockStore:
     async def count_uncompactified_blocks(self) -> int:
         if self.db_wrapper.db_version == 2:
             # DB V2 has an index on is_fully_compactified only for blocks in the main chain
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute(
                     "select count(*) from full_blocks where is_fully_compactified=0 and in_main_chain=1"
                 ) as cursor:
                     row = await cursor.fetchone()
         else:
-            async with self.db_wrapper.read_db() as conn:
+            async with self.db_wrapper.reader_no_transaction() as conn:
                 async with conn.execute("select count(*) from full_blocks where is_fully_compactified=0") as cursor:
                     row = await cursor.fetchone()
 
