@@ -19,7 +19,7 @@ from chia.plotting.util import (
     remove_plot,
     remove_plot_directory,
 )
-from chia.util.streamable import dataclass_from_dict
+from chia.server.server import ChiaServer
 
 log = logging.getLogger(__name__)
 
@@ -35,6 +35,7 @@ class Harvester:
     constants: ConsensusConstants
     _refresh_lock: asyncio.Lock
     event_loop: asyncio.events.AbstractEventLoop
+    server: Optional[ChiaServer]
 
     def __init__(self, root_path: Path, config: Dict, constants: ConsensusConstants):
         self.log = log
@@ -50,7 +51,9 @@ class Harvester:
                 refresh_parameter, interval_seconds=config["plot_loading_frequency_seconds"]
             )
         if "plots_refresh_parameter" in config:
-            refresh_parameter = dataclass_from_dict(PlotsRefreshParameter, config["plots_refresh_parameter"])
+            refresh_parameter = PlotsRefreshParameter.from_json_dict(config["plots_refresh_parameter"])
+
+        self.log.info(f"Using plots_refresh_parameter: {refresh_parameter}")
 
         self.plot_manager = PlotManager(
             root_path, refresh_parameter=refresh_parameter, refresh_callback=self._plot_refresh_callback
