@@ -7,40 +7,53 @@ from typing_extensions import Protocol
 from chia.farmer.farmer import Farmer
 from chia.plot_sync.receiver import Receiver
 from chia.protocols.harvester_protocol import Plot
+from chia.rpc.rpc_server import Endpoint, EndpointResult
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.byte_types import hexstr_to_bytes
+from chia.util.ints import uint32
 from chia.util.paginator import Paginator
-from chia.util.streamable import dataclass_from_dict
+from chia.util.streamable import Streamable, streamable
 from chia.util.ws_message import WsRpcMessage, create_payload_dict
 
 
 class PaginatedRequestData(Protocol):
-    node_id: bytes32
-    page: int
-    page_size: int
+    @property
+    def node_id(self) -> bytes32:
+        pass
+
+    @property
+    def page(self) -> uint32:
+        pass
+
+    @property
+    def page_size(self) -> uint32:
+        pass
 
 
-@dataclasses.dataclass
-class FilterItem:
+@streamable
+@dataclasses.dataclass(frozen=True)
+class FilterItem(Streamable):
     key: str
     value: Optional[str]
 
 
-@dataclasses.dataclass
-class PlotInfoRequestData:
+@streamable
+@dataclasses.dataclass(frozen=True)
+class PlotInfoRequestData(Streamable):
     node_id: bytes32
-    page: int
-    page_size: int
+    page: uint32
+    page_size: uint32
     filter: List[FilterItem] = dataclasses.field(default_factory=list)
     sort_key: str = "filename"
     reverse: bool = False
 
 
-@dataclasses.dataclass
-class PlotPathRequestData:
+@streamable
+@dataclasses.dataclass(frozen=True)
+class PlotPathRequestData(Streamable):
     node_id: bytes32
-    page: int
-    page_size: int
+    page: uint32
+    page_size: uint32
     filter: List[str] = dataclasses.field(default_factory=list)
     reverse: bool = False
 
@@ -69,7 +82,7 @@ class FarmerRpcApi:
         self.service = farmer
         self.service_name = "chia_farmer"
 
-    def get_routes(self) -> Dict[str, Callable]:
+    def get_routes(self) -> Dict[str, Endpoint]:
         return {
             "/get_signage_point": self.get_signage_point,
             "/get_signage_points": self.get_signage_points,
@@ -148,7 +161,7 @@ class FarmerRpcApi:
 
         return payloads
 
-    async def get_signage_point(self, request: Dict) -> Dict:
+    async def get_signage_point(self, request: Dict) -> EndpointResult:
         sp_hash = hexstr_to_bytes(request["sp_hash"])
         for _, sps in self.service.sps.items():
             for sp in sps:
@@ -167,7 +180,7 @@ class FarmerRpcApi:
                     }
         raise ValueError(f"Signage point {sp_hash.hex()} not found")
 
-    async def get_signage_points(self, _: Dict) -> Dict[str, Any]:
+    async def get_signage_points(self, _: Dict) -> EndpointResult:
         result: List[Dict[str, Any]] = []
         for sps in self.service.sps.values():
             for sp in sps:
@@ -187,12 +200,12 @@ class FarmerRpcApi:
                 )
         return {"signage_points": result}
 
-    async def get_reward_targets(self, request: Dict) -> Dict:
+    async def get_reward_targets(self, request: Dict) -> EndpointResult:
         search_for_private_key = request["search_for_private_key"]
         max_ph_to_search = request.get("max_ph_to_search", 500)
         return await self.service.get_reward_targets(search_for_private_key, max_ph_to_search)
 
-    async def set_reward_targets(self, request: Dict) -> Dict:
+    async def set_reward_targets(self, request: Dict) -> EndpointResult:
         farmer_target, pool_target = None, None
         if "farmer_target" in request:
             farmer_target = request["farmer_target"]
@@ -210,7 +223,7 @@ class FarmerRpcApi:
             )
         return plot_count
 
-    async def get_pool_state(self, _: Dict) -> Dict:
+    async def get_pool_state(self, _: Dict) -> EndpointResult:
         pools_list = []
         for p2_singleton_puzzle_hash, pool_dict in self.service.pool_state.items():
             pool_state = pool_dict.copy()
@@ -219,20 +232,20 @@ class FarmerRpcApi:
             pools_list.append(pool_state)
         return {"pool_state": pools_list}
 
-    async def set_payout_instructions(self, request: Dict) -> Dict:
+    async def set_payout_instructions(self, request: Dict) -> EndpointResult:
         launcher_id: bytes32 = bytes32.from_hexstr(request["launcher_id"])
         await self.service.set_payout_instructions(launcher_id, request["payout_instructions"])
         return {}
 
-    async def get_harvesters(self, _: Dict):
+    async def get_harvesters(self, _: Dict) -> EndpointResult:
         return await self.service.get_harvesters(False)
 
-    async def get_harvesters_summary(self, _: Dict[str, object]) -> Dict[str, object]:
+    async def get_harvesters_summary(self, _: Dict[str, object]) -> EndpointResult:
         return await self.service.get_harvesters(True)
 
-    async def get_harvester_plots_valid(self, request_dict: Dict[str, object]) -> Dict[str, object]:
+    async def get_harvester_plots_valid(self, request_dict: Dict[str, object]) -> EndpointResult:
         # TODO: Consider having a extra List[PlotInfo] in Receiver to avoid rebuilding the list for each call
-        request = dataclass_from_dict(PlotInfoRequestData, request_dict)
+        request = PlotInfoRequestData.from_json_dict(request_dict)
         plot_list = list(self.service.get_receiver(request.node_id).plots().values())
         # Apply filter
         plot_list = [
@@ -249,26 +262,25 @@ class FarmerRpcApi:
     def paginated_plot_path_request(
         self, source_func: Callable[[Receiver], List[str]], request_dict: Dict[str, object]
     ) -> Dict[str, object]:
-        request: PlotPathRequestData = dataclass_from_dict(PlotPathRequestData, request_dict)
+        request: PlotPathRequestData = PlotPathRequestData.from_json_dict(request_dict)
         receiver = self.service.get_receiver(request.node_id)
         source = source_func(receiver)
-        request = dataclass_from_dict(PlotPathRequestData, request_dict)
         # Apply filter
         source = [plot for plot in source if all(filter_item in plot for filter_item in request.filter)]
         # Apply reverse
         source = sorted(source, reverse=request.reverse)
         return paginated_plot_request(source, request)
 
-    async def get_harvester_plots_invalid(self, request_dict: Dict[str, object]) -> Dict[str, object]:
+    async def get_harvester_plots_invalid(self, request_dict: Dict[str, object]) -> EndpointResult:
         return self.paginated_plot_path_request(Receiver.invalid, request_dict)
 
-    async def get_harvester_plots_keys_missing(self, request_dict: Dict[str, object]) -> Dict[str, object]:
+    async def get_harvester_plots_keys_missing(self, request_dict: Dict[str, object]) -> EndpointResult:
         return self.paginated_plot_path_request(Receiver.keys_missing, request_dict)
 
-    async def get_harvester_plots_duplicates(self, request_dict: Dict[str, object]) -> Dict[str, object]:
+    async def get_harvester_plots_duplicates(self, request_dict: Dict[str, object]) -> EndpointResult:
         return self.paginated_plot_path_request(Receiver.duplicates, request_dict)
 
-    async def get_pool_login_link(self, request: Dict) -> Dict:
+    async def get_pool_login_link(self, request: Dict) -> EndpointResult:
         launcher_id: bytes32 = bytes32(hexstr_to_bytes(request["launcher_id"]))
         login_link: Optional[str] = await self.service.generate_login_link(launcher_id)
         if login_link is None:
