@@ -1,5 +1,6 @@
-from typing import Any, Callable, Dict, List, Optional
+from typing import Any, Dict, List, Optional
 
+from chia.rpc.rpc_server import Endpoint
 from chia.timelord.timelord import Timelord
 from chia.util.ws_message import WsRpcMessage, create_payload_dict
 
@@ -9,7 +10,7 @@ class TimelordRpcApi:
         self.service = timelord
         self.service_name = "chia_timelord"
 
-    def get_routes(self) -> Dict[str, Callable]:
+    def get_routes(self) -> Dict[str, Endpoint]:
         return {}
 
     async def _state_changed(self, change: str, change_data: Optional[Dict[str, Any]] = None) -> List[WsRpcMessage]:
