@@ -1,6 +1,6 @@
 from dataclasses import dataclass
 from enum import IntEnum
-from typing import Any, Optional
+from typing import Optional, SupportsBytes, Union
 
 from chia.protocols.protocol_message_types import ProtocolMessageTypes
 from chia.util.ints import uint8, uint16
@@ -14,6 +14,7 @@ class NodeType(IntEnum):
     TIMELORD = 4
     INTRODUCER = 5
     WALLET = 6
+    DATA_LAYER = 7
 
 
 class Delivery(IntEnum):
@@ -41,5 +42,5 @@ class Message(Streamable):
     data: bytes
 
 
-def make_msg(msg_type: ProtocolMessageTypes, data: Any) -> Message:
+def make_msg(msg_type: ProtocolMessageTypes, data: Union[bytes, SupportsBytes]) -> Message:
     return Message(uint8(msg_type.value), None, bytes(data))
