@@ -1,4 +1,3 @@
-import dataclasses
 import warnings
 
 from dataclasses import dataclass
@@ -9,7 +8,7 @@ from blspy import AugSchemeMPL, G2Element
 from chia.consensus.default_constants import DEFAULT_CONSTANTS
 from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
-from chia.util.streamable import Streamable, dataclass_from_dict, recurse_jsonify, streamable
+from chia.util.streamable import Streamable, streamable_from_dict, recurse_jsonify, streamable
 from chia.wallet.util.debug_spend_bundle import debug_spend_bundle
 
 from .coin_spend import CoinSpend
@@ -96,12 +95,12 @@ class SpendBundle(Streamable):
                 warnings.warn("`coin_solutions` is now `coin_spends` in `SpendBundle.from_json_dict`")
             else:
                 raise ValueError("JSON contains both `coin_solutions` and `coin_spends`, just use `coin_spends`")
-        return dataclass_from_dict(cls, json_dict)
+        return streamable_from_dict(cls, json_dict)
 
     def to_json_dict(self, include_legacy_keys: bool = True, exclude_modern_keys: bool = True):
         if include_legacy_keys is False and exclude_modern_keys is True:
             raise ValueError("`coin_spends` not included in legacy or modern outputs")
-        d = dataclasses.asdict(self)
+        d = recurse_jsonify(self)
         if include_legacy_keys:
             d["coin_solutions"] = d["coin_spends"]
         if exclude_modern_keys:
