@@ -2,7 +2,6 @@
 
 import click
 import colorama
-import threading
 import yaml
 
 from chia.cmds.passphrase_funcs import prompt_for_passphrase, read_passphrase_from_file
@@ -17,14 +16,6 @@ from typing import Any, Dict, Optional
 DEFAULT_KEYRING_YAML = DEFAULT_KEYS_ROOT_PATH / "keyring.yaml"
 
 
-class DumpKeyring(FileKeyring):  # lgtm [py/missing-call-to-init]
-    def __init__(self, keyring_file: Path):
-        self.keyring_path = keyring_file
-        self.payload_cache = {}
-        self.load_keyring_lock = threading.RLock()
-        # We don't call super().__init__() to avoid side-effects
-
-
 def get_passphrase_prompt(keyring_file: str) -> str:
     prompt = (
         colorama.Fore.YELLOW
@@ -60,7 +51,8 @@ def dump(keyring_file, full_payload: bool, passphrase_file: Optional[TextIOWrapp
     if passphrase_file is not None:
         passphrase = read_passphrase_from_file(passphrase_file)
 
-    keyring = DumpKeyring(Path(keyring_file))
+    keyring_path = Path(keyring_file)
+    keyring = FileKeyring(keyring_path)
 
     if full_payload:
         keyring.load_outer_payload()
@@ -86,47 +78,6 @@ def dump(keyring_file, full_payload: bool, passphrase_file: Optional[TextIOWrapp
             break
 
 
-def dump_to_string(
-    keyring_file, full_payload: bool, passphrase_file: Optional[TextIOWrapper], pretty_print: bool
-) -> str:
-    saved_passphrase: Optional[str] = KeyringWrapper.get_shared_instance().get_master_passphrase_from_credential_store()
-    passphrase: str = saved_passphrase or DEFAULT_PASSPHRASE_IF_NO_MASTER_PASSPHRASE
-    prompt: str = get_passphrase_prompt(str(keyring_file))
-    data: Dict[str, Any] = {}
-
-    print(f"Attempting to dump contents of keyring file: {keyring_file}\n")
-
-    if passphrase_file is not None:
-        passphrase = read_passphrase_from_file(passphrase_file)
-
-    keyring = DumpKeyring(Path(keyring_file))
-
-    if full_payload:
-        keyring.load_outer_payload()
-        data = keyring.outer_payload_cache
-
-    s: str = ""
-    for i in range(5):
-        try:
-            keyring.load_keyring(passphrase)
-            if len(data) > 0:
-                data["data"] = keyring.payload_cache
-            else:
-                data = keyring.payload_cache
-
-            if pretty_print:
-                s = yaml.dump(data)
-            else:
-                s = str(data)
-            break
-        except (ValueError, InvalidTag):
-            passphrase = prompt_for_passphrase(prompt)
-        except Exception as e:
-            print(f"Unhandled exception: {e}")
-            break
-    return s
-
-
 def main():
     colorama.init()
     dump()  # pylint: disable=no-value-for-parameter
