@@ -99,10 +99,6 @@ class Node(metaclass=ABCMeta):
     def add(self, toadd: bytes, depth: int) -> "Node":
         pass
 
-    @abstractmethod
-    def remove(self, toremove: bytes, depth: int):
-        pass
-
     @abstractmethod
     def is_included(self, tocheck: bytes, depth: int, p: List[bytes]) -> bool:
         pass
@@ -131,9 +127,6 @@ class MerkleSet:
     def add_already_hashed(self, toadd: bytes):
         self.root = self.root.add(toadd, 0)
 
-    def remove_already_hashed(self, toremove: bytes):
-        self.root = self.root.remove(toremove, 0)
-
     def is_included_already_hashed(self, tocheck: bytes) -> Tuple[bool, bytes]:
         proof: List = []
         r = self.root.is_included(tocheck, 0, proof)
@@ -164,9 +157,6 @@ class EmptyNode(Node):
     def add(self, toadd: bytes, depth: int) -> Node:
         return TerminalNode(toadd)
 
-    def remove(self, toremove: bytes, depth: int) -> Node:
-        return self
-
     def is_included(self, tocheck: bytes, depth: int, p: List[bytes]) -> bool:
         p.append(EMPTY)
         return False
@@ -181,6 +171,16 @@ class EmptyNode(Node):
 _empty = EmptyNode()
 
 
+def _make_middle(children: Any, depth: int) -> Node:
+    cbits = [get_bit(child.hash, depth) for child in children]
+    if cbits[0] != cbits[1]:
+        return MiddleNode(children)
+    nextvals: List[Node] = [_empty, _empty]
+    nextvals[cbits[0] ^ 1] = _empty
+    nextvals[cbits[0]] = _make_middle(children, depth + 1)
+    return MiddleNode(nextvals)
+
+
 class TerminalNode(Node):
     def __init__(self, hash: bytes, bits: List[int] = None):
         assert len(hash) == 32
@@ -204,23 +204,9 @@ class TerminalNode(Node):
         if toadd == self.hash:
             return self
         if toadd > self.hash:
-            return self._make_middle([self, TerminalNode(toadd)], depth)
+            return _make_middle([self, TerminalNode(toadd)], depth)
         else:
-            return self._make_middle([TerminalNode(toadd), self], depth)
-
-    def _make_middle(self, children: Any, depth: int) -> Node:
-        cbits = [get_bit(child.hash, depth) for child in children]
-        if cbits[0] != cbits[1]:
-            return MiddleNode(children)
-        nextvals: List[Node] = [_empty, _empty]
-        nextvals[cbits[0] ^ 1] = _empty
-        nextvals[cbits[0]] = self._make_middle(children, depth + 1)
-        return MiddleNode(nextvals)
-
-    def remove(self, toremove: bytes, depth: int) -> Node:
-        if toremove == self.hash:
-            return _empty
-        return self
+            return _make_middle([TerminalNode(toadd), self], depth)
 
     def is_included(self, tocheck: bytes, depth: int, p: List[bytes]) -> bool:
         p.append(TERMINAL + self.hash)
@@ -277,21 +263,6 @@ class MiddleNode(Node):
         newvals[bit] = newchild
         return MiddleNode(newvals)
 
-    def remove(self, toremove: bytes, depth: int) -> Node:
-        bit = get_bit(toremove, depth)
-        child = self.children[bit]
-        newchild = child.remove(toremove, depth + 1)
-        if newchild is child:
-            return self
-        otherchild = self.children[bit ^ 1]
-        if newchild.is_empty() and otherchild.is_terminal():
-            return otherchild
-        if newchild.is_terminal() and otherchild.is_empty():
-            return newchild
-        newvals = [x for x in self.children]
-        newvals[bit] = newchild
-        return MiddleNode(newvals)
-
     def is_included(self, tocheck: bytes, depth: int, p: List[bytes]) -> bool:
         p.append(MIDDLE)
         if get_bit(tocheck, depth) == 0:
@@ -332,9 +303,6 @@ class TruncatedNode(Node):
     def add(self, toadd: bytes, depth: int) -> Node:
         return self
 
-    def remove(self, toremove: bytes, depth: int) -> Node:
-        return self
-
     def is_included(self, tocheck: bytes, depth: int, p: List[bytes]) -> bool:
         raise SetError()
 
@@ -349,18 +317,10 @@ class SetError(Exception):
     pass
 
 
-def confirm_included(root: bytes32, val: bytes, proof: bytes32) -> bool:
-    return confirm_not_included_already_hashed(root, sha256(val).digest(), proof)
-
-
 def confirm_included_already_hashed(root: bytes32, val: bytes, proof: bytes) -> bool:
     return _confirm(root, val, proof, True)
 
 
-def confirm_not_included(root: bytes32, val: bytes, proof: bytes32) -> bool:
-    return confirm_not_included_already_hashed(root, sha256(val).digest(), proof)
-
-
 def confirm_not_included_already_hashed(root: bytes32, val: bytes, proof: bytes) -> bool:
     return _confirm(root, val, proof, False)
 
