@@ -7,7 +7,6 @@ from typing import Any, Dict, List, Optional, Set, Tuple, Type, TypeVar
 from blspy import AugSchemeMPL, G2Element
 
 from chia.protocols.wallet_protocol import CoinState
-from chia.server.outbound_message import NodeType
 from chia.server.ws_connection import WSChiaConnection
 from chia.types.announcement import Announcement
 from chia.types.blockchain_format.coin import Coin
@@ -63,7 +62,6 @@ class NFTWallet:
         wallet: Wallet,
         did_id: Optional[bytes32] = None,
         name: Optional[str] = None,
-        in_transaction: bool = False,
     ) -> _T_NFTWallet:
         """
         This must be called under the wallet state manager lock
@@ -77,20 +75,13 @@ class NFTWallet:
         self.nft_wallet_info = NFTWalletInfo(did_id)
         self.my_nft_coins = []
         info_as_string = json.dumps(self.nft_wallet_info.to_json_dict())
-        wallet_info = await wallet_state_manager.user_store.create_wallet(
-            name,
-            uint32(WalletType.NFT.value),
-            info_as_string,
-            in_transaction=in_transaction,
+        self.wallet_info = await wallet_state_manager.user_store.create_wallet(
+            name, uint32(WalletType.NFT.value), info_as_string
         )
-
-        if wallet_info is None:
-            raise ValueError("Internal Error")
-        self.wallet_info = wallet_info
         self.wallet_id = self.wallet_info.id
         self.log.debug("NFT wallet id: %r and standard wallet id: %r", self.wallet_id, self.standard_wallet.wallet_id)
 
-        await self.wallet_state_manager.add_new_wallet(self, self.wallet_info.id, in_transaction=in_transaction)
+        await self.wallet_state_manager.add_new_wallet(self, self.wallet_info.id)
         self.log.debug("Generated a new NFT wallet: %s", self.__dict__)
         return self
 
@@ -148,43 +139,33 @@ class NFTWallet:
                 return nft_coin
         raise KeyError(f"Couldn't find coin with id: {nft_coin_id}")
 
-    async def add_nft_coin(self, coin: Coin, spent_height: uint32, in_transaction: bool) -> None:
-        await self.coin_added(coin, spent_height, in_transaction=in_transaction)
-
-    async def coin_added(self, coin: Coin, height: uint32, in_transaction: bool) -> None:
+    async def coin_added(self, coin: Coin, height: uint32, peer: WSChiaConnection) -> None:
         """Notification from wallet state manager that wallet has been received."""
         self.log.info(f"NFT wallet %s has been notified that {coin} was added", self.wallet_info.name)
         for coin_info in self.my_nft_coins:
             if coin_info.coin == coin:
                 return
         wallet_node = self.wallet_state_manager.wallet_node
-        server = wallet_node.server
-        full_nodes: Dict[bytes32, WSChiaConnection] = server.connection_by_type.get(NodeType.FULL_NODE, {})
         cs: Optional[CoinSpend] = None
-        coin_states: Optional[List[CoinState]] = await self.wallet_state_manager.wallet_node.get_coin_state(
-            [coin.parent_coin_info]
-        )
+        coin_states: Optional[List[CoinState]] = await wallet_node.get_coin_state([coin.parent_coin_info], peer=peer)
         if not coin_states:
             # farm coin
             return
         assert coin_states
         parent_coin = coin_states[0].coin
-        for node_id in full_nodes:
-            node = server.all_connections[node_id]
-            cs = await wallet_node.fetch_puzzle_solution(node, height, parent_coin)
-            if cs is not None:
-                break
+        cs = await wallet_node.fetch_puzzle_solution(height, parent_coin, peer)
         assert cs is not None
-        await self.puzzle_solution_received(cs, in_transaction=in_transaction)
+        await self.puzzle_solution_received(cs, peer)
 
-    async def puzzle_solution_received(self, coin_spend: CoinSpend, in_transaction: bool) -> None:
+    async def puzzle_solution_received(self, coin_spend: CoinSpend, peer: WSChiaConnection) -> None:
         self.log.debug("Puzzle solution received to wallet: %s", self.wallet_info)
         coin_name = coin_spend.coin.name()
         puzzle: Program = Program.from_bytes(bytes(coin_spend.puzzle_reveal))
         # At this point, the puzzle must be a NFT puzzle.
         # This method will be called only when the wallet state manager uncurried this coin as a NFT puzzle.
 
-        uncurried_nft = UncurriedNFT.uncurry(puzzle)
+        uncurried_nft = UncurriedNFT.uncurry(*puzzle.uncurry())
+        assert uncurried_nft is not None
         self.log.info(
             f"found the info for NFT coin {coin_name} {uncurried_nft.inner_puzzle} {uncurried_nft.singleton_struct}"
         )
@@ -229,7 +210,7 @@ class NFTWallet:
         else:
             raise ValueError("Couldn't generate child puzzle for NFT")
         launcher_coin_states: List[CoinState] = await self.wallet_state_manager.wallet_node.get_coin_state(
-            [singleton_id]
+            [singleton_id], peer=peer
         )
         assert (
             launcher_coin_states is not None
@@ -241,25 +222,24 @@ class NFTWallet:
 
         # all is well, lets add NFT to our local db
         parent_coin = None
-        coin_record = await self.wallet_state_manager.coin_store.get_coin_record(coin_name)
-        if coin_record is None:
-            coin_states: Optional[List[CoinState]] = await self.wallet_state_manager.wallet_node.get_coin_state(
-                [coin_name]
-            )
-            if coin_states is not None:
-                parent_coin = coin_states[0].coin
-        if coin_record is not None:
-            parent_coin = coin_record.coin
-        if parent_coin is None:
+        confirmed_height = None
+        coin_states: Optional[List[CoinState]] = await self.wallet_state_manager.wallet_node.get_coin_state(
+            [coin_name], peer=peer
+        )
+        if coin_states is not None:
+            parent_coin = coin_states[0].coin
+            confirmed_height = coin_states[0].spent_height
+
+        if parent_coin is None or confirmed_height is None:
             raise ValueError("Error finding parent")
 
         await self.add_coin(
             child_coin,
             singleton_id,
             child_puzzle,
-            LineageProof(parent_coin.parent_coin_info, parent_inner_puzhash, parent_coin.amount),
+            LineageProof(parent_coin.parent_coin_info, parent_inner_puzhash, uint64(parent_coin.amount)),
             mint_height,
-            in_transaction=in_transaction,
+            confirmed_height,
         )
 
     async def add_coin(
@@ -269,27 +249,25 @@ class NFTWallet:
         puzzle: Program,
         lineage_proof: LineageProof,
         mint_height: uint32,
-        in_transaction: bool,
+        confirmed_height: uint32,
     ) -> None:
         my_nft_coins = self.my_nft_coins
         for coin_info in my_nft_coins:
             if coin_info.coin == coin:
                 my_nft_coins.remove(coin_info)
-        new_nft = NFTCoinInfo(nft_id, coin, lineage_proof, puzzle, mint_height)
+        new_nft = NFTCoinInfo(nft_id, coin, lineage_proof, puzzle, mint_height, confirmed_height)
         my_nft_coins.append(new_nft)
-        await self.wallet_state_manager.nft_store.save_nft(
-            self.id(), self.get_did(), new_nft, in_transaction=in_transaction
-        )
-        await self.wallet_state_manager.add_interested_coin_ids([coin.name()], in_transaction=in_transaction)
+        await self.wallet_state_manager.nft_store.save_nft(self.id(), self.get_did(), new_nft)
+        await self.wallet_state_manager.add_interested_coin_ids([coin.name()])
         self.wallet_state_manager.state_changed("nft_coin_added", self.wallet_info.id)
         return
 
-    async def remove_coin(self, coin: Coin, in_transaction: bool) -> None:
+    async def remove_coin(self, coin: Coin, height: uint32) -> None:
         my_nft_coins = self.my_nft_coins
         for coin_info in my_nft_coins:
             if coin_info.coin == coin:
                 my_nft_coins.remove(coin_info)
-                await self.wallet_state_manager.nft_store.delete_nft(coin_info.nft_id, in_transaction=in_transaction)
+                await self.wallet_state_manager.nft_store.delete_nft(coin_info.nft_id, height)
         self.wallet_state_manager.state_changed("nft_coin_removed", self.wallet_info.id)
         return
 
@@ -408,13 +386,13 @@ class NFTWallet:
         nft_coin = NFTCoinInfo(
             nft_id=launcher_coin.name(),
             coin=eve_coin,
-            lineage_proof=LineageProof(parent_name=launcher_coin.parent_coin_info, amount=launcher_coin.amount),
+            lineage_proof=LineageProof(parent_name=launcher_coin.parent_coin_info, amount=uint64(launcher_coin.amount)),
             full_puzzle=eve_fullpuz,
             mint_height=uint32(0),
         )
         # Don't set fee, it is covered in the tx_record
         txs = await self.generate_signed_transaction(
-            [eve_coin.amount],
+            [uint64(eve_coin.amount)],
             [target_puzzle_hash],
             nft_coin=nft_coin,
             new_owner=did_id,
@@ -435,12 +413,8 @@ class NFTWallet:
         for spend in spend_bundle.coin_spends:
             pks = {}
             if not puzzle_hashes:
-                try:
-                    uncurried_nft = UncurriedNFT.uncurry(spend.puzzle_reveal.to_program())
-                except ValueError:
-                    # not an NFT
-                    pass
-                else:
+                uncurried_nft = UncurriedNFT.uncurry(*spend.puzzle_reveal.to_program().uncurry())
+                if uncurried_nft is not None:
                     self.log.debug("Found a NFT state layer to sign")
                     puzzle_hashes.append(uncurried_nft.p2_puzzle.get_tree_hash())
             for ph in puzzle_hashes:
@@ -475,7 +449,8 @@ class NFTWallet:
     async def update_metadata(
         self, nft_coin_info: NFTCoinInfo, key: str, uri: str, fee: uint64 = uint64(0)
     ) -> Optional[SpendBundle]:
-        uncurried_nft = UncurriedNFT.uncurry(nft_coin_info.full_puzzle)
+        uncurried_nft = UncurriedNFT.uncurry(*nft_coin_info.full_puzzle.uncurry())
+        assert uncurried_nft is not None
         puzzle_hash = uncurried_nft.p2_puzzle.get_tree_hash()
 
         self.log.info(
@@ -495,9 +470,11 @@ class NFTWallet:
     def get_current_nfts(self) -> List[NFTCoinInfo]:
         return self.my_nft_coins
 
-    async def update_coin_status(
-        self, coin_id: bytes32, pending_transaction: bool, in_transaction: bool = False
-    ) -> None:
+    async def load_current_nft(self) -> List[NFTCoinInfo]:
+        self.my_nft_coins = await self.wallet_state_manager.nft_store.get_nft_list(wallet_id=self.wallet_id)
+        return self.my_nft_coins
+
+    async def update_coin_status(self, coin_id: bytes32, pending_transaction: bool) -> None:
         my_nft_coins = self.my_nft_coins
         target_nft: Optional[NFTCoinInfo] = None
         for coin_info in my_nft_coins:
@@ -512,20 +489,19 @@ class NFTWallet:
             target_nft.lineage_proof,
             target_nft.full_puzzle,
             target_nft.mint_height,
+            target_nft.latest_height,
             pending_transaction,
         )
         my_nft_coins.append(new_nft)
-        await self.wallet_state_manager.nft_store.save_nft(
-            self.id(), self.get_did(), new_nft, in_transaction=in_transaction
-        )
+        await self.wallet_state_manager.nft_store.save_nft(self.id(), self.get_did(), new_nft)
 
-    async def save_info(self, nft_info: NFTWalletInfo, in_transaction: bool) -> None:
+    async def save_info(self, nft_info: NFTWalletInfo) -> None:
         self.nft_wallet_info = nft_info
         current_info = self.wallet_info
         data_str = json.dumps(nft_info.to_json_dict())
         wallet_info = WalletInfo(current_info.id, current_info.name, current_info.type, data_str)
         self.wallet_info = wallet_info
-        await self.wallet_state_manager.user_store.update_wallet(wallet_info, in_transaction)
+        await self.wallet_state_manager.user_store.update_wallet(wallet_info)
 
     async def convert_puzzle_hash(self, puzhash: bytes32) -> bytes32:
         return puzhash
@@ -536,7 +512,7 @@ class NFTWallet:
                 return coin
         return None
 
-    def get_puzzle_info(self, nft_id: bytes32) -> PuzzleInfo:
+    async def get_puzzle_info(self, nft_id: bytes32) -> PuzzleInfo:
         nft_coin: Optional[NFTCoinInfo] = self.get_nft(nft_id)
         if nft_coin is None:
             raise ValueError("An asset ID was specified that this wallet doesn't track")
@@ -546,11 +522,13 @@ class NFTWallet:
         else:
             return puzzle_info
 
-    async def get_coins_to_offer(self, nft_id: bytes32, amount: uint64) -> Set[Coin]:
+    async def get_coins_to_offer(
+        self, nft_id: bytes32, amount: uint64, min_coin_amount: Optional[uint64] = None
+    ) -> Set[Coin]:
         nft_coin: Optional[NFTCoinInfo] = self.get_nft(nft_id)
         if nft_coin is None:
             raise ValueError("An asset ID was specified that this wallet doesn't track")
-        return set([nft_coin.coin])
+        return {nft_coin.coin}
 
     def match_puzzle_info(self, puzzle_driver: PuzzleInfo) -> bool:
         return (
@@ -568,7 +546,6 @@ class NFTWallet:
         wallet: Wallet,
         puzzle_driver: PuzzleInfo,
         name=None,
-        in_transaction=False,
     ) -> Any:
         # Off the bat we don't support multiple profile but when we do this will have to change
         for wallet in wallet_state_manager.wallets.values():
@@ -581,7 +558,6 @@ class NFTWallet:
             wallet,
             None,
             name,
-            in_transaction,
         )
 
     async def create_tandem_xch_tx(
@@ -723,7 +699,8 @@ class NFTWallet:
             puzzle_announcements_to_assert=puzzle_announcements_bytes,
         )
 
-        unft = UncurriedNFT.uncurry(nft_coin.full_puzzle)
+        unft = UncurriedNFT.uncurry(*nft_coin.full_puzzle.uncurry())
+        assert unft is not None
         magic_condition = None
         if unft.supports_did:
             if new_owner is None:
@@ -760,32 +737,28 @@ class NFTWallet:
         offer_dict: Dict[Optional[bytes32], int],
         driver_dict: Dict[bytes32, PuzzleInfo],
         fee: uint64,
+        min_coin_amount: Optional[uint64] = None,
     ) -> Offer:
-        amounts = list(offer_dict.values())
+        amounts = list(offer_dict.values())  # Note: this does not include the royalties
         if len(offer_dict) != 2 or (amounts[0] > 0 == amounts[1] > 0):
             raise ValueError("Royalty enabled NFTs only support offering/requesting one NFT for one currency")
 
-        first_asset_id = list(offer_dict.items())[0][0]
-        if first_asset_id is None:
-            nft: bool = False
-        else:
-            nft = driver_dict[first_asset_id].check_type(
-                [
-                    AssetType.SINGLETON.value,
-                    AssetType.METADATA.value,
-                    AssetType.OWNERSHIP.value,
-                ]
-            )
+        offering_nft: bool = False
+        for asset_id, amount in offer_dict.items():
+            if amount < 0:  # negative means offering
+                offered_asset_id: Optional[bytes32] = asset_id
+                if asset_id is not None:
+                    offering_nft = driver_dict[asset_id].check_type(  # check if asset is an NFT
+                        [
+                            AssetType.SINGLETON.value,
+                            AssetType.METADATA.value,
+                            AssetType.OWNERSHIP.value,
+                        ]
+                    )
+            else:
+                requested_asset_id: Optional[bytes32] = asset_id
 
-        offered: bool = list(offer_dict.items())[0][1] < 0
-        if offered:
-            offered_asset_id: Optional[bytes32] = first_asset_id
-            requested_asset_id: Optional[bytes32] = list(offer_dict.items())[1][0]
-        else:
-            offered_asset_id = list(offer_dict.items())[1][0]
-            requested_asset_id = first_asset_id
-
-        if nft == offered:
+        if offering_nft:  # if we are offering an NFT.
             assert offered_asset_id is not None  # hello mypy
             driver_dict[offered_asset_id].info["also"]["also"]["owner"] = "()"
             wallet = await wallet_state_manager.get_wallet_for_asset_id(offered_asset_id.hex())
@@ -794,7 +767,7 @@ class NFTWallet:
             offered_coin_info = wallet.get_nft(offered_asset_id)
             offered_coin: Coin = offered_coin_info.coin
             requested_amount = offer_dict[requested_asset_id]
-            if requested_asset_id is None:
+            if requested_asset_id is None:  # If we are just asking for xch.
                 trade_prices = Program.to([[uint64(requested_amount), OFFER_MOD.get_tree_hash()]])
             else:
                 trade_prices = Program.to(
@@ -821,8 +794,7 @@ class NFTWallet:
             total_spend_bundle = SpendBundle.aggregate(transaction_bundles)
 
             return Offer(notarized_payments, total_spend_bundle, driver_dict)
-        else:
-            assert isinstance(requested_asset_id, bytes32)
+        elif isinstance(requested_asset_id, bytes32):  # if we are requesting an NFT.
             driver_dict[requested_asset_id].info["also"]["also"]["owner"] = "()"
             requested_info = driver_dict[requested_asset_id]
             transfer_info = requested_info.also().also()  # type: ignore
@@ -836,20 +808,18 @@ class NFTWallet:
             offered_amount = uint64(abs(offer_dict[offered_asset_id]))
             royalty_amount = uint64(offered_amount * royalty_percentage / 10000)
             if offered_amount == royalty_amount:
+                # Due to the coin id's matching this is impossible unless we use separate coins
+                # when fulfilling the offer.
                 raise ValueError("Amount offered and amount paid in royalties are equal")
-            if offered_asset_id is None:
-                # std xch offer
+            if offered_asset_id is None:  # xch offer
                 wallet = wallet_state_manager.main_wallet
+                coin_amount_needed: int = offered_amount + royalty_amount + fee
             else:
                 # cat offer
                 wallet = await wallet_state_manager.get_wallet_for_asset_id(offered_asset_id.hex())
-
-            if wallet.type() == WalletType.STANDARD_WALLET:
-                coin_amount_needed: int = offered_amount + royalty_amount + fee
-            else:
                 coin_amount_needed = offered_amount + royalty_amount
-            pmt_coins = list(await wallet.get_coins_to_offer(offered_asset_id, coin_amount_needed))
 
+            pmt_coins = list(await wallet.get_coins_to_offer(offered_asset_id, coin_amount_needed, min_coin_amount))
             notarized_payments = Offer.notarize_payments(requested_payments, pmt_coins)
             announcements_to_assert = Offer.calculate_announcements(notarized_payments, driver_dict)
             # Calculate the royalty announcement separately
@@ -863,7 +833,6 @@ class NFTWallet:
                     driver_dict,
                 )
             )
-
             if wallet.type() == WalletType.STANDARD_WALLET:
                 tx = await wallet.generate_signed_transaction(
                     offered_amount,
@@ -887,27 +856,29 @@ class NFTWallet:
             txn_bundles: List[SpendBundle] = [tx.spend_bundle for tx in all_transactions if tx.spend_bundle is not None]
             txn_spend_bundle = SpendBundle.aggregate(txn_bundles)
             # Create a spend bundle for the royalty payout from OFFER MOD
-            for txn in txn_bundles:
-                for coin in txn.additions():
-                    if coin.amount == royalty_amount:
-                        royalty_coin = coin
-                        parent_spend = txn.coin_spends[0]
-                        break
-            assert royalty_coin
             # make the royalty payment solution
             # ((nft_launcher_id . ((ROYALTY_ADDRESS, royalty_amount, (ROYALTY_ADDRESS)))))
+            # we are basically just recreating the royalty announcement above.
             inner_royalty_sol = Program.to([[requested_asset_id, [royalty_address, royalty_amount, [royalty_address]]]])
-            if offered_asset_id is None:
+            if offered_asset_id is None:  # xch offer
                 offer_puzzle: Program = OFFER_MOD
                 royalty_sol = inner_royalty_sol
-            else:
+            else:  # CAT
                 offer_puzzle = construct_puzzle(driver_dict[offered_asset_id], OFFER_MOD)
+            royalty_ph = offer_puzzle.get_tree_hash()
+            for txn in txn_bundles:
+                for coin in txn.additions():
+                    if coin.amount == royalty_amount and coin.puzzle_hash == royalty_ph:
+                        royalty_coin = coin
+                        parent_spend = txn.coin_spends[0]
+            assert royalty_coin
+            if offered_asset_id is not None:  # if CAT
                 #  adapt royalty_sol to work with cat puzzle
                 royalty_coin_hex = (
                     "0x"
                     + royalty_coin.parent_coin_info.hex()
                     + royalty_coin.puzzle_hash.hex()
-                    + bytes(royalty_coin.amount).hex()
+                    + bytes(uint64(royalty_coin.amount)).hex()
                 )
                 parent_spend_hex: str = "0x" + bytes(parent_spend).hex()
                 solver = Solver(
@@ -921,15 +892,18 @@ class NFTWallet:
                     }
                 )
                 royalty_sol = solve_puzzle(driver_dict[offered_asset_id], solver, OFFER_MOD, inner_royalty_sol)
-            royalty_spend = SpendBundle([CoinSpend(royalty_coin, offer_puzzle, royalty_sol)], G2Element())
 
+            royalty_spend = SpendBundle([CoinSpend(royalty_coin, offer_puzzle, royalty_sol)], G2Element())
             total_spend_bundle = SpendBundle.aggregate([txn_spend_bundle, royalty_spend])
             offer = Offer(notarized_payments, total_spend_bundle, driver_dict)
             return offer
+        else:
+            raise ValueError("No NFT in offer!")
 
     async def set_nft_did(self, nft_coin_info: NFTCoinInfo, did_id: bytes, fee: uint64 = uint64(0)) -> SpendBundle:
         self.log.debug("Setting NFT DID with parameters: nft=%s did=%s", nft_coin_info, did_id)
-        unft = UncurriedNFT.uncurry(nft_coin_info.full_puzzle)
+        unft = UncurriedNFT.uncurry(*nft_coin_info.full_puzzle.uncurry())
+        assert unft is not None
         nft_id = unft.singleton_launcher_id
         puzzle_hashes_to_sign = [unft.p2_puzzle.get_tree_hash()]
         did_inner_hash = b""
@@ -939,7 +913,7 @@ class NFTWallet:
             additional_bundles.append(did_bundle)
 
         nft_tx_record = await self.generate_signed_transaction(
-            [nft_coin_info.coin.amount],
+            [uint64(nft_coin_info.coin.amount)],
             puzzle_hashes_to_sign,
             fee,
             {nft_coin_info.coin},
