@@ -6,7 +6,7 @@ from clvm_tools.binutils import disassemble
 
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.blockchain_format.coin import Coin, coin_as_list
-from chia.types.blockchain_format.program import Program
+from chia.types.blockchain_format.program import Program, INFINITE_COST
 from chia.types.announcement import Announcement
 from chia.types.coin_spend import CoinSpend
 from chia.types.spend_bundle import SpendBundle
@@ -33,6 +33,19 @@ OFFER_MOD = load_clvm("settlement_payments.clvm")
 ZERO_32 = bytes32([0] * 32)
 
 
+def detect_dependent_coin(
+    names: List[bytes32], deps: Dict[bytes32, List[bytes32]], announcement_dict: Dict[bytes32, List[bytes32]]
+) -> Optional[Tuple[bytes32, bytes32]]:
+    # First, we check for any dependencies on coins in the same bundle
+    for name in names:
+        for dependency in deps[name]:
+            for coin, announces in announcement_dict.items():
+                if dependency in announces and coin != name:
+                    # We found one, now remove it and anything that depends on it (except the "provider")
+                    return name, coin
+    return None
+
+
 @dataclass(frozen=True)
 class NotarizedPayment(Payment):
     nonce: bytes32 = ZERO_32
@@ -96,13 +109,6 @@ class Offer:
         return announcements
 
     def __post_init__(self):
-        # Verify that there is at least something being offered
-        offered_coins: Dict[bytes32, List[Coin]] = self.get_offered_coins()
-        if offered_coins == {}:
-            raise ValueError("Bundle is not offering anything")
-        if self.get_requested_payments() == {}:
-            raise ValueError("Bundle is not requesting anything")
-
         # Verify that there are no duplicate payments
         for payments in self.requested_payments.values():
             payment_programs: List[bytes32] = [p.name() for p in payments]
@@ -155,22 +161,23 @@ class Offer:
                 inner_solution: Optional[Program] = get_inner_solution(puzzle_driver, parent_solution)
                 assert inner_puzzle is not None and inner_solution is not None
                 conditions: Program = inner_puzzle.run(inner_solution)
+                matching_spend_additions: List[Coin] = []  # coins that match offered amount and are sent to offer ph.
                 for condition in conditions.as_iter():
                     if condition.first() == 51 and condition.rest().first() == OFFER_HASH:
-                        additions_w_amount: List[Coin] = [
-                            a for a in additions if a.amount == condition.rest().rest().first().as_int()
-                        ]
-                        if len(additions_w_amount) == 1:
-                            coins_for_this_spend.append(additions_w_amount[0])
-                        else:
-                            additions_w_amount_and_puzhash: List[Coin] = [
-                                a
-                                for a in additions_w_amount
-                                if a.puzzle_hash
-                                == construct_puzzle(puzzle_driver, OFFER_HASH).get_tree_hash(OFFER_HASH)  # type: ignore
-                            ]
-                            if len(additions_w_amount_and_puzhash) == 1:
-                                coins_for_this_spend.append(additions_w_amount_and_puzhash[0])
+                        matching_spend_additions.extend(
+                            [a for a in additions if a.amount == condition.rest().rest().first().as_int()]
+                        )
+                if len(matching_spend_additions) == 1:
+                    coins_for_this_spend.append(matching_spend_additions[0])
+                else:
+                    additions_w_amount_and_puzhash: List[Coin] = [
+                        a
+                        for a in matching_spend_additions
+                        if a.puzzle_hash
+                        == construct_puzzle(puzzle_driver, OFFER_HASH).get_tree_hash(OFFER_HASH)  # type: ignore
+                    ]
+                    if len(additions_w_amount_and_puzhash) == 1:
+                        coins_for_this_spend.append(additions_w_amount_and_puzhash[0])
             else:
                 asset_id = None
                 coins_for_this_spend.extend([a for a in additions if a.puzzle_hash == OFFER_HASH])
@@ -198,6 +205,11 @@ class Offer:
         return requested_amounts
 
     def arbitrage(self) -> Dict[Optional[bytes32], int]:
+        """
+        Returns a dictionary of the type of each asset and amount that is involved in the trade
+        With the amount being how much their offered amount within the offer
+        exceeds/falls short of their requested amount.
+        """
         offered_amounts: Dict[Optional[bytes32], int] = self.get_offered_amounts()
         requested_amounts: Dict[Optional[bytes32], int] = self.get_requested_amounts()
 
@@ -282,6 +294,51 @@ class Offer:
                 primary_coins.add(self.get_root_removal(coin))
         return list(primary_coins)
 
+    # This returns the minimum coins that when spent will invalidate the rest of the bundle
+    def get_cancellation_coins(self) -> List[Coin]:
+        # First, we're going to gather:
+        dependencies: Dict[bytes32, List[bytes32]] = {}  # all of the hashes that each coin depends on
+        announcements: Dict[bytes32, List[bytes32]] = {}  # all of the hashes of the announcement that each coin makes
+        coin_names: List[bytes32] = []  # The names of all the coins
+        for spend in [cs for cs in self.bundle.coin_spends if cs.coin not in self.bundle.additions()]:
+            name = bytes32(spend.coin.name())
+            coin_names.append(name)
+            dependencies[name] = []
+            announcements[name] = []
+            conditions: Program = spend.puzzle_reveal.run_with_cost(INFINITE_COST, spend.solution)[1]
+            for condition in conditions.as_iter():
+                if condition.first() == 60:  # create coin announcement
+                    announcements[name].append(Announcement(name, condition.at("rf").as_python()).name())
+                elif condition.first() == 61:  # assert coin announcement
+                    dependencies[name].append(bytes32(condition.at("rf").as_python()))
+
+        # We now enter a loop that is attempting to express the following logic:
+        # "If I am depending on another coin in the same bundle, you may as well cancel that coin instead of me"
+        # By the end of the loop, we should have filtered down the list of coin_names to include only those that will
+        # cancel everything else
+        while True:
+            removed = detect_dependent_coin(coin_names, dependencies, announcements)
+            if removed is None:
+                break
+            removed_coin, provider = removed
+            removed_announcements: List[bytes32] = announcements[removed_coin]
+            remove_these_keys: List[bytes32] = [removed_coin]
+            while True:
+                for coin, deps in dependencies.items():
+                    if set(deps) & set(removed_announcements) and coin != provider:
+                        remove_these_keys.append(coin)
+                removed_announcements = []
+                for coin in remove_these_keys:
+                    dependencies.pop(coin)
+                    removed_announcements.extend(announcements.pop(coin))
+                coin_names = [n for n in coin_names if n not in remove_these_keys]
+                if removed_announcements == []:
+                    break
+                else:
+                    remove_these_keys = []
+
+        return [cs.coin for cs in self.bundle.coin_spends if cs.coin.name() in coin_names]
+
     @classmethod
     def aggregate(cls, offers: List["Offer"]) -> "Offer":
         total_requested_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
@@ -321,11 +378,13 @@ class Offer:
             raise ValueError("Offer is currently incomplete")
 
         completion_spends: List[CoinSpend] = []
+        all_offered_coins: Dict[Optional[bytes32], List[Coin]] = self.get_offered_coins()
+        total_arbitrage_amount: Dict[Optional[bytes32], int] = self.arbitrage()
         for asset_id, payments in self.requested_payments.items():
-            offered_coins: List[Coin] = self.get_offered_coins()[asset_id]
+            offered_coins: List[Coin] = all_offered_coins[asset_id]
 
             # Because of CAT supply laws, we must specify a place for the leftovers to go
-            arbitrage_amount: int = self.arbitrage()[asset_id]
+            arbitrage_amount: int = total_arbitrage_amount[asset_id]
             all_payments: List[NotarizedPayment] = payments.copy()
             if arbitrage_amount > 0:
                 assert arbitrage_amount is not None
@@ -362,7 +421,7 @@ class Offer:
                                 "0x"
                                 + sibling_coin.parent_coin_info.hex()
                                 + sibling_coin.puzzle_hash.hex()
-                                + bytes(sibling_coin.amount).hex()
+                                + bytes(uint64(sibling_coin.amount)).hex()
                             )
                             sibling_spends += "0x" + bytes(coin_to_spend_dict[sibling_coin]).hex() + ")"
                             sibling_puzzles += disassembled_offer_mod
@@ -379,7 +438,7 @@ class Offer:
                                 "coin": "0x"
                                 + coin.parent_coin_info.hex()
                                 + coin.puzzle_hash.hex()
-                                + bytes(coin.amount).hex(),
+                                + bytes(uint64(coin.amount)).hex(),
                                 "parent_spend": "0x" + bytes(coin_to_spend_dict[coin]).hex(),
                                 "siblings": siblings,
                                 "sibling_spends": sibling_spends,
