@@ -6,7 +6,7 @@ import logging
 
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.mempool_inclusion_status import MempoolInclusionStatus
-from chia.util.db_wrapper import DBWrapper
+from chia.util.db_wrapper import DBWrapper2
 from chia.util.errors import Err
 from chia.util.ints import uint8, uint32
 from chia.wallet.trade_record import TradeRecord
@@ -46,13 +46,14 @@ class TradeStore:
     TradeStore stores trading history.
     """
 
-    db_connection: aiosqlite.Connection
     cache_size: uint32
-    db_wrapper: DBWrapper
+    db_wrapper: DBWrapper2
     log: logging.Logger
 
     @classmethod
-    async def create(cls, db_wrapper: DBWrapper, cache_size: uint32 = uint32(600000), name: str = None) -> "TradeStore":
+    async def create(
+        cls, db_wrapper: DBWrapper2, cache_size: uint32 = uint32(600000), name: str = None
+    ) -> "TradeStore":
         self = cls()
 
         if name:
@@ -62,54 +63,44 @@ class TradeStore:
 
         self.cache_size = cache_size
         self.db_wrapper = db_wrapper
-        self.db_connection = db_wrapper.db
-        await self.db_connection.execute(
-            (
-                "CREATE TABLE IF NOT EXISTS trade_records("
-                " trade_record blob,"
-                " trade_id text PRIMARY KEY,"
-                " status int,"
-                " confirmed_at_index int,"
-                " created_at_time bigint,"
-                " sent int,"
-                " is_my_offer tinyint)"
+
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
+            await conn.execute(
+                (
+                    "CREATE TABLE IF NOT EXISTS trade_records("
+                    " trade_record blob,"
+                    " trade_id text PRIMARY KEY,"
+                    " status int,"
+                    " confirmed_at_index int,"
+                    " created_at_time bigint,"
+                    " sent int,"
+                    " is_my_offer tinyint)"
+                )
             )
-        )
 
-        # Attempt to add the is_my_offer column. If successful, migrate is_my_offer to the new column.
-        needs_is_my_offer_migration: bool = False
-        try:
-            await self.db_connection.execute("ALTER TABLE trade_records ADD COLUMN is_my_offer tinyint")
-            needs_is_my_offer_migration = True
-        except aiosqlite.OperationalError:
-            pass  # ignore what is likely Duplicate column error
+            # Attempt to add the is_my_offer column. If successful, migrate is_my_offer to the new column.
+            needs_is_my_offer_migration: bool = False
+            try:
+                await conn.execute("ALTER TABLE trade_records ADD COLUMN is_my_offer tinyint")
+                needs_is_my_offer_migration = True
+            except aiosqlite.OperationalError:
+                pass  # ignore what is likely Duplicate column error
 
-        await self.db_connection.execute(
-            "CREATE INDEX IF NOT EXISTS trade_confirmed_index on trade_records(confirmed_at_index)"
-        )
-        await self.db_connection.execute("CREATE INDEX IF NOT EXISTS trade_status on trade_records(status)")
-        await self.db_connection.execute("CREATE INDEX IF NOT EXISTS trade_id on trade_records(trade_id)")
+            await conn.execute("CREATE INDEX IF NOT EXISTS trade_confirmed_index on trade_records(confirmed_at_index)")
+            await conn.execute("CREATE INDEX IF NOT EXISTS trade_status on trade_records(status)")
+            await conn.execute("CREATE INDEX IF NOT EXISTS trade_id on trade_records(trade_id)")
 
-        await self.db_connection.commit()
-
-        if needs_is_my_offer_migration:
-            await migrate_is_my_offer(self.log, self.db_connection)
+            if needs_is_my_offer_migration:
+                await migrate_is_my_offer(self.log, conn)
 
         return self
 
-    async def _clear_database(self):
-        cursor = await self.db_connection.execute("DELETE FROM trade_records")
-        await cursor.close()
-        await self.db_connection.commit()
-
-    async def add_trade_record(self, record: TradeRecord, in_transaction) -> None:
+    async def add_trade_record(self, record: TradeRecord) -> None:
         """
         Store TradeRecord into DB
         """
-        if not in_transaction:
-            await self.db_wrapper.lock.acquire()
-        try:
-            cursor = await self.db_connection.execute(
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
+            cursor = await conn.execute(
                 "INSERT OR REPLACE INTO trade_records "
                 "(trade_record, trade_id, status, confirmed_at_index, created_at_time, sent, is_my_offer) "
                 "VALUES(?, ?, ?, ?, ?, ?, ?)",
@@ -124,12 +115,8 @@ class TradeStore:
                 ),
             )
             await cursor.close()
-        finally:
-            if not in_transaction:
-                await self.db_connection.commit()
-                self.db_wrapper.lock.release()
 
-    async def set_status(self, trade_id: bytes32, status: TradeStatus, in_transaction: bool, index: uint32 = uint32(0)):
+    async def set_status(self, trade_id: bytes32, status: TradeStatus, index: uint32 = uint32(0)):
         """
         Updates the status of the trade
         """
@@ -152,7 +139,7 @@ class TradeStore:
             status=uint32(status.value),
             sent_to=current.sent_to,
         )
-        await self.add_trade_record(tx, in_transaction)
+        await self.add_trade_record(tx)
 
     async def increment_sent(
         self, id: bytes32, name: str, send_status: MempoolInclusionStatus, err: Optional[Err]
@@ -190,7 +177,7 @@ class TradeStore:
             sent_to=sent_to,
         )
 
-        await self.add_trade_record(tx, False)
+        await self.add_trade_record(tx)
         return True
 
     async def get_trades_count(self) -> Tuple[int, int, int]:
@@ -201,9 +188,11 @@ class TradeStore:
         query += "SUM(CASE WHEN is_my_offer=1 THEN 1 ELSE 0 END) AS my_offers, "
         query += "SUM(CASE WHEN is_my_offer=0 THEN 1 ELSE 0 END) AS taken_offers "
         query += "FROM trade_records"
-        cursor = await self.db_connection.execute(query)
-        row = await cursor.fetchone()
-        await cursor.close()
+
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute(query)
+            row = await cursor.fetchone()
+            await cursor.close()
 
         total = 0
         my_offers_count = 0
@@ -223,9 +212,10 @@ class TradeStore:
         """
         Checks DB for TradeRecord with id: id and returns it.
         """
-        cursor = await self.db_connection.execute("SELECT * from trade_records WHERE trade_id=?", (trade_id.hex(),))
-        row = await cursor.fetchone()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute("SELECT trade_record from trade_records WHERE trade_id=?", (trade_id.hex(),))
+            row = await cursor.fetchone()
+            await cursor.close()
         if row is not None:
             record = TradeRecord.from_bytes(row[0])
             return record
@@ -235,9 +225,10 @@ class TradeStore:
         """
         Checks DB for TradeRecord with id: id and returns it.
         """
-        cursor = await self.db_connection.execute("SELECT * from trade_records WHERE status=?", (status.value,))
-        rows = await cursor.fetchall()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute("SELECT trade_record from trade_records WHERE status=?", (status.value,))
+            rows = await cursor.fetchall()
+            await cursor.close()
         records = []
         for row in rows:
             record = TradeRecord.from_bytes(row[0])
@@ -250,9 +241,10 @@ class TradeStore:
         Returns the list of trades that have not been received by full node yet.
         """
 
-        cursor = await self.db_connection.execute("SELECT * from trade_records WHERE sent<? and confirmed=?", (4, 0))
-        rows = await cursor.fetchall()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute("SELECT trade_record from trade_records WHERE sent<? and confirmed=?", (4, 0))
+            rows = await cursor.fetchall()
+            await cursor.close()
         records = []
         for row in rows:
             record = TradeRecord.from_bytes(row[0])
@@ -265,9 +257,10 @@ class TradeStore:
         Returns the list of all trades that have not yet been confirmed.
         """
 
-        cursor = await self.db_connection.execute("SELECT * from trade_records WHERE confirmed=?", (0,))
-        rows = await cursor.fetchall()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute("SELECT trade_record from trade_records WHERE confirmed=?", (0,))
+            rows = await cursor.fetchall()
+            await cursor.close()
         records = []
 
         for row in rows:
@@ -281,9 +274,10 @@ class TradeStore:
         Returns all stored trades.
         """
 
-        cursor = await self.db_connection.execute("SELECT * from trade_records")
-        rows = await cursor.fetchall()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute("SELECT trade_record from trade_records")
+            rows = await cursor.fetchall()
+            await cursor.close()
         records = []
 
         for row in rows:
@@ -362,7 +356,7 @@ class TradeStore:
         else:
             raise ValueError(f"No known sort {sort_key}")
 
-        query = "SELECT * from trade_records "
+        query = "SELECT trade_record FROM trade_records "
         args = []
 
         if exclude_my_offers or exclude_taken_offers:
@@ -375,7 +369,7 @@ class TradeStore:
             if where_status_clause is not None:
                 query += "AND " + where_status_clause
         else:
-            query = "SELECT * from trade_records "
+            query = "SELECT trade_record FROM trade_records "
             # Include the additional WHERE status clause if we're filtering out certain statuses
             if where_status_clause is not None:
                 query += "WHERE " + where_status_clause
@@ -388,9 +382,10 @@ class TradeStore:
 
         args.extend([limit, offset])
 
-        cursor = await self.db_connection.execute(query, tuple(args))
-        rows = await cursor.fetchall()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute(query, tuple(args))
+            rows = await cursor.fetchall()
+            await cursor.close()
 
         records = []
 
@@ -401,9 +396,10 @@ class TradeStore:
         return records
 
     async def get_trades_above(self, height: uint32) -> List[TradeRecord]:
-        cursor = await self.db_connection.execute("SELECT * from trade_records WHERE confirmed_at_index>?", (height,))
-        rows = await cursor.fetchall()
-        await cursor.close()
+        async with self.db_wrapper.reader_no_transaction() as conn:
+            cursor = await conn.execute("SELECT trade_record from trade_records WHERE confirmed_at_index>?", (height,))
+            rows = await cursor.fetchall()
+            await cursor.close()
         records = []
 
         for row in rows:
@@ -413,10 +409,8 @@ class TradeStore:
         return records
 
     async def rollback_to_block(self, block_index):
-        async with self.db_wrapper.lock:
+
+        async with self.db_wrapper.writer_maybe_transaction() as conn:
             # Delete from storage
-            cursor = await self.db_connection.execute(
-                "DELETE FROM trade_records WHERE confirmed_at_index>?", (block_index,)
-            )
+            cursor = await conn.execute("DELETE FROM trade_records WHERE confirmed_at_index>?", (block_index,))
             await cursor.close()
-            await self.db_connection.commit()
