@@ -1,24 +1,30 @@
 import asyncio
+
+from chia.protocols.shared_protocol import Capability
 import logging
 import random
 from typing import List, Optional, Tuple, Union, Dict
+
 from chia_rs import compute_merkle_set_root
 
 from chia.consensus.constants import ConsensusConstants
 from chia.protocols import wallet_protocol
 from chia.protocols.wallet_protocol import (
     RequestAdditions,
+    RequestBlockHeaders,
     RespondAdditions,
     RejectAdditionsRequest,
     RejectRemovalsRequest,
+    RespondBlockHeaders,
     RespondRemovals,
     RequestRemovals,
-    RespondBlockHeader,
     CoinState,
     RespondToPhUpdates,
     RespondToCoinUpdates,
     RespondHeaderBlocks,
     RequestHeaderBlocks,
+    RejectHeaderBlocks,
+    RejectBlockHeaders,
 )
 from chia.server.ws_connection import WSChiaConnection
 from chia.types.blockchain_format.coin import hash_coin_ids, Coin
@@ -37,11 +43,12 @@ async def fetch_last_tx_from_peer(height: uint32, peer: WSChiaConnection) -> Opt
     while True:
         if request_height == -1:
             return None
-        request = wallet_protocol.RequestBlockHeader(uint32(request_height))
-        response: Optional[RespondBlockHeader] = await peer.request_block_header(request)
-        if response is not None and isinstance(response, RespondBlockHeader):
-            if response.header_block.is_transaction_block:
-                return response.header_block
+        response: Optional[List[HeaderBlock]] = await request_header_blocks(
+            peer, uint32(request_height), uint32(request_height)
+        )
+        if response is not None and len(response) > 0:
+            if response[0].is_transaction_block:
+                return response[0]
         elif request_height < height:
             # The peer might be slightly behind others but still synced, so we should allow fetching one more TX block
             break
@@ -152,7 +159,7 @@ def validate_removals(
         # Verify removals root
         removals_merkle_set = MerkleSet()
         for name_coin in coins:
-            name, coin = name_coin
+            _, coin = name_coin
             if coin is not None:
                 removals_merkle_set.add_already_hashed(coin.name())
         removals_root = removals_merkle_set.get_root()
@@ -206,20 +213,28 @@ async def request_and_validate_removals(
 
 
 async def request_and_validate_additions(
-    peer: WSChiaConnection, height: uint32, header_hash: bytes32, puzzle_hash: bytes32, additions_root: bytes32
-):
+    peer: WSChiaConnection,
+    peer_request_cache: PeerRequestCache,
+    height: uint32,
+    header_hash: bytes32,
+    puzzle_hash: bytes32,
+    additions_root: bytes32,
+) -> bool:
+    if peer_request_cache.in_additions_in_block(header_hash, puzzle_hash):
+        return True
     additions_request = RequestAdditions(height, header_hash, [puzzle_hash])
     additions_res: Optional[Union[RespondAdditions, RejectAdditionsRequest]] = await peer.request_additions(
         additions_request
     )
     if additions_res is None or isinstance(additions_res, RejectAdditionsRequest):
         return False
-
-    return validate_additions(
+    result: bool = validate_additions(
         additions_res.coins,
         additions_res.proofs,
         additions_root,
     )
+    peer_request_cache.add_to_additions_in_block(header_hash, puzzle_hash, height)
+    return result
 
 
 def get_block_challenge(
@@ -289,25 +304,59 @@ def last_change_height_cs(cs: CoinState) -> uint32:
     return uint32(0)
 
 
+def get_block_header(block):
+    return HeaderBlock(
+        block.finished_sub_slots,
+        block.reward_chain_block,
+        block.challenge_chain_sp_proof,
+        block.challenge_chain_ip_proof,
+        block.reward_chain_sp_proof,
+        block.reward_chain_ip_proof,
+        block.infused_challenge_chain_ip_proof,
+        block.foliage,
+        block.foliage_transaction_block,
+        b"",  # we don't need the filter
+        block.transactions_info,
+    )
+
+
+async def request_header_blocks(
+    peer: WSChiaConnection, start_height: uint32, end_height: uint32
+) -> Optional[List[HeaderBlock]]:
+    if Capability.BLOCK_HEADERS in peer.peer_capabilities:
+        response = await peer.request_block_headers(RequestBlockHeaders(start_height, end_height, False))
+    else:
+        response = await peer.request_header_blocks(RequestHeaderBlocks(start_height, end_height))
+    if response is None or isinstance(response, RejectBlockHeaders) or isinstance(response, RejectHeaderBlocks):
+        return None
+    return response.header_blocks
+
+
 async def _fetch_header_blocks_inner(
-    all_peers: List[WSChiaConnection],
-    request: RequestHeaderBlocks,
-) -> Optional[RespondHeaderBlocks]:
+    all_peers: List[Tuple[WSChiaConnection, bool]],
+    request_start: uint32,
+    request_end: uint32,
+) -> Optional[Union[RespondHeaderBlocks, RespondBlockHeaders]]:
     # We will modify this list, don't modify passed parameters.
-    remaining_peers = list(all_peers)
+    bytes_api_peers = [peer for peer in all_peers if Capability.BLOCK_HEADERS in peer[0].peer_capabilities]
+    other_peers = [peer for peer in all_peers if Capability.BLOCK_HEADERS not in peer[0].peer_capabilities]
+    random.shuffle(bytes_api_peers)
+    random.shuffle(other_peers)
 
-    while len(remaining_peers) > 0:
-        peer = random.choice(remaining_peers)
+    for peer, is_trusted in bytes_api_peers + other_peers:
+        if Capability.BLOCK_HEADERS in peer.peer_capabilities:
+            response = await peer.request_block_headers(RequestBlockHeaders(request_start, request_end, False))
+        else:
+            response = await peer.request_header_blocks(RequestHeaderBlocks(request_start, request_end))
 
-        response = await peer.request_header_blocks(request)
-
-        if isinstance(response, RespondHeaderBlocks):
+        if isinstance(response, (RespondHeaderBlocks, RespondBlockHeaders)):
             return response
 
         # Request to peer failed in some way, close the connection and remove the peer
         # from our local list.
-        await peer.close()
-        remaining_peers.remove(peer)
+        if not is_trusted:
+            log.info(f"Closing peer {peer} since it does not have the blocks we asked for")
+            await peer.close()
 
     return None
 
@@ -316,7 +365,7 @@ async def fetch_header_blocks_in_range(
     start: uint32,
     end: uint32,
     peer_request_cache: PeerRequestCache,
-    all_peers: List[WSChiaConnection],
+    all_peers: List[Tuple[WSChiaConnection, bool]],
 ) -> Optional[List[HeaderBlock]]:
     blocks: List[HeaderBlock] = []
     for i in range(start - (start % 32), end + 1, 32):
@@ -327,13 +376,12 @@ async def fetch_header_blocks_in_range(
         if res_h_blocks_task is not None:
             log.debug(f"Using cache for: {start}-{end}")
             if res_h_blocks_task.done():
-                res_h_blocks: Optional[RespondHeaderBlocks] = res_h_blocks_task.result()
+                res_h_blocks: Optional[Union[RespondBlockHeaders, RespondHeaderBlocks]] = res_h_blocks_task.result()
             else:
                 res_h_blocks = await res_h_blocks_task
         else:
             log.debug(f"Fetching: {start}-{end}")
-            request_header_blocks = RequestHeaderBlocks(request_start, request_end)
-            res_h_blocks_task = asyncio.create_task(_fetch_header_blocks_inner(all_peers, request_header_blocks))
+            res_h_blocks_task = asyncio.create_task(_fetch_header_blocks_inner(all_peers, request_start, request_end))
             peer_request_cache.add_to_block_requests(request_start, request_end, res_h_blocks_task)
             res_h_blocks = await res_h_blocks_task
         if res_h_blocks is None:
