@@ -20,6 +20,8 @@ class WalletType(IntEnum):
     DECENTRALIZED_ID = 8
     POOLING_WALLET = 9
     NFT = 10
+    DATA_LAYER = 11
+    DATA_LAYER_OFFER = 12
 
 
 class AmountWithPuzzlehash(TypedDict):
