@@ -1,14 +1,20 @@
+from os import remove
 from pathlib import Path
+from tempfile import NamedTemporaryFile
 from unittest import TestCase
 
+import pytest
 from clvm_tools.clvmc import compile_clvm
 
 from chia.types.blockchain_format.program import Program, SerializedProgram
 
+pytestmark = pytest.mark.data_layer
+
+
 wallet_program_files = set(
     [
         "chia/wallet/puzzles/calculate_synthetic_public_key.clvm",
-        "chia/wallet/puzzles/cat.clvm",
+        "chia/wallet/puzzles/cat_v2.clvm",
         "chia/wallet/puzzles/chialisp_deserialisation.clvm",
         "chia/wallet/puzzles/rom_bootstrap_generator.clvm",
         "chia/wallet/puzzles/generator_for_single_coin.clvm",
@@ -46,6 +52,9 @@ wallet_program_files = set(
         "chia/wallet/puzzles/nft_state_layer.clvm",
         "chia/wallet/puzzles/nft_ownership_layer.clvm",
         "chia/wallet/puzzles/nft_ownership_transfer_program_one_way_claim_with_royalties.clvm",
+        "chia/wallet/puzzles/graftroot_dl_offers.clvm",
+        "chia/wallet/puzzles/p2_parent.clvm",
+        "chia/wallet/puzzles/decompress_block_spends.clvm",
     ]
 )
 
@@ -167,3 +176,13 @@ class TestClvmCompilation(TestCase):
                 existing_sha,
                 msg=f"Checked-in shatree hash file does not match shatree hash of loaded Program: {prog_path}",
             )
+
+    def test_017_encoding_bug_fixed(self):
+        with NamedTemporaryFile(delete=False) as tf:
+            tf.write(b"10000000")
+        hexname = tf.name + ".hex"
+        compile_clvm(tf.name, hexname, [])
+        with open(hexname) as f:
+            self.assertEqual(f.read().strip(), "8400989680")
+        remove(tf.name)
+        remove(hexname)
