@@ -15,7 +15,7 @@ from chia.util.hash import std_hash
 from chia.util.ints import uint16
 from tests.core.node_height import node_height_exactly, node_height_between
 from tests.setup_nodes import test_constants
-from tests.time_out_assert import time_out_assert
+from chia.simulator.time_out_assert import time_out_assert
 
 log = logging.getLogger(__name__)
 
@@ -44,9 +44,11 @@ class TestFullSync:
             PeerInfo(self_hostname, uint16(server_1._port)), on_connect=full_node_2.full_node.on_connect
         )
 
+        timeout_seconds = 250
+
         # The second node should eventually catch up to the first one
         await time_out_assert(
-            150, node_height_exactly, True, full_node_2, test_constants.WEIGHT_PROOF_RECENT_BLOCKS - 5 - 1
+            timeout_seconds, node_height_exactly, True, full_node_2, test_constants.WEIGHT_PROOF_RECENT_BLOCKS - 5 - 1
         )
 
         for block in blocks[
@@ -58,8 +60,6 @@ class TestFullSync:
             PeerInfo(self_hostname, uint16(server_1._port)), on_connect=full_node_3.full_node.on_connect
         )
 
-        timeout_seconds = 150
-
         # Node 3 and Node 2 sync up to node 1
         await time_out_assert(
             timeout_seconds, node_height_exactly, True, full_node_2, test_constants.WEIGHT_PROOF_RECENT_BLOCKS + 5 - 1
@@ -157,7 +157,7 @@ class TestFullSync:
 
         # The second node should eventually catch up to the first one, and have the
         # same tip at height num_blocks - 1
-        await time_out_assert(180, node_height_exactly, True, full_node_2, num_blocks_initial - 1)
+        await time_out_assert(300, node_height_exactly, True, full_node_2, num_blocks_initial - 1)
         await time_out_assert(180, node_height_exactly, True, full_node_3, num_blocks_initial - 1)
 
         def fn3_is_not_syncing():
@@ -174,7 +174,7 @@ class TestFullSync:
         log.warning(f"FN3 height {full_node_3.full_node.blockchain.get_peak().height}")
 
         # TODO: fix this flaky test
-        await time_out_assert(120, node_height_exactly, True, full_node_3, 999)
+        await time_out_assert(180, node_height_exactly, True, full_node_3, 999)
 
         await server_2.start_client(PeerInfo(self_hostname, uint16(server_1._port)), full_node_2.full_node.on_connect)
         await server_3.start_client(PeerInfo(self_hostname, uint16(server_1._port)), full_node_3.full_node.on_connect)
@@ -183,13 +183,13 @@ class TestFullSync:
         await time_out_assert(180, node_height_exactly, True, full_node_2, 999)
 
     @pytest.mark.asyncio
-    async def test_batch_sync(self, two_nodes, bt, self_hostname):
+    async def test_batch_sync(self, two_nodes, self_hostname):
         # Must be below "sync_block_behind_threshold" in the config
         num_blocks = 20
         num_blocks_2 = 9
+        full_node_1, full_node_2, server_1, server_2, bt = two_nodes
         blocks = bt.get_consecutive_blocks(num_blocks)
         blocks_2 = bt.get_consecutive_blocks(num_blocks_2, seed=b"123")
-        full_node_1, full_node_2, server_1, server_2 = two_nodes
 
         # 12 blocks to node_1
         for block in blocks:
@@ -206,11 +206,11 @@ class TestFullSync:
         await time_out_assert(60, node_height_exactly, True, full_node_2, num_blocks - 1)
 
     @pytest.mark.asyncio
-    async def test_backtrack_sync_1(self, two_nodes, bt, self_hostname):
+    async def test_backtrack_sync_1(self, two_nodes, self_hostname):
+        full_node_1, full_node_2, server_1, server_2, bt = two_nodes
         blocks = bt.get_consecutive_blocks(1, skip_slots=1)
         blocks = bt.get_consecutive_blocks(1, blocks, skip_slots=0)
         blocks = bt.get_consecutive_blocks(1, blocks, skip_slots=0)
-        full_node_1, full_node_2, server_1, server_2 = two_nodes
 
         # 3 blocks to node_1 in different sub slots
         for block in blocks:
@@ -223,10 +223,10 @@ class TestFullSync:
         await time_out_assert(60, node_height_exactly, True, full_node_2, 2)
 
     @pytest.mark.asyncio
-    async def test_backtrack_sync_2(self, two_nodes, bt, self_hostname):
+    async def test_backtrack_sync_2(self, two_nodes, self_hostname):
+        full_node_1, full_node_2, server_1, server_2, bt = two_nodes
         blocks = bt.get_consecutive_blocks(1, skip_slots=3)
         blocks = bt.get_consecutive_blocks(8, blocks, skip_slots=0)
-        full_node_1, full_node_2, server_1, server_2 = two_nodes
 
         # 3 blocks to node_1 in different sub slots
         for block in blocks:
@@ -324,7 +324,7 @@ class TestFullSync:
 
     @pytest.mark.asyncio
     async def test_block_ses_mismatch(self, two_nodes, default_1000_blocks, self_hostname):
-        full_node_1, full_node_2, server_1, server_2 = two_nodes
+        full_node_1, full_node_2, server_1, server_2, _ = two_nodes
         blocks = default_1000_blocks
 
         for block in blocks[:501]:
