@@ -11,7 +11,7 @@ from chia.types.condition_with_args import ConditionWithArgs
 from chia.types.spend_bundle import SpendBundle
 from chia.util.ints import uint64
 from tests.util.db_connection import DBConnection
-from tests.wallet_tools import WalletTool
+from chia.simulator.wallet_tools import WalletTool
 
 
 log = logging.getLogger(__name__)
@@ -92,7 +92,7 @@ class TestHintStore:
             coins_for_hint_0 = await hint_store.get_coin_ids(hint_0)
             assert coin_id_0 in coins_for_hint_0
 
-            async with db_wrapper.read_db() as conn:
+            async with db_wrapper.reader_no_transaction() as conn:
                 cursor = await conn.execute("SELECT COUNT(*) FROM hints")
                 rows = await cursor.fetchall()
 
@@ -105,8 +105,8 @@ class TestHintStore:
                 assert rows[0][0] == 4
 
     @pytest.mark.asyncio
-    async def test_hints_in_blockchain(self, bt, wallet_nodes):  # noqa: F811
-        full_node_1, full_node_2, server_1, server_2, wallet_a, wallet_receiver = wallet_nodes
+    async def test_hints_in_blockchain(self, wallet_nodes):  # noqa: F811
+        full_node_1, full_node_2, server_1, server_2, wallet_a, wallet_receiver, bt = wallet_nodes
 
         blocks = bt.get_consecutive_blocks(
             5,
