@@ -9,16 +9,16 @@ from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate
 from chia.full_node.full_node_api import FullNodeAPI
 from chia.protocols import full_node_protocol
 from chia.simulator.simulator_protocol import FarmNewBlockProtocol
+from chia.simulator.time_out_assert import time_out_assert
 from chia.types.peer_info import PeerInfo
 from chia.util.ints import uint16, uint32
-from tests.time_out_assert import time_out_assert
 
 
 class TestTransactions:
     @pytest.mark.asyncio
     async def test_wallet_coinbase(self, wallet_node_sim_and_wallet, self_hostname):
         num_blocks = 5
-        full_nodes, wallets = wallet_node_sim_and_wallet
+        full_nodes, wallets, _ = wallet_node_sim_and_wallet
         full_node_api = full_nodes[0]
         full_node_server = full_node_api.server
         wallet_node, server_2 = wallets[0]
@@ -35,12 +35,12 @@ class TestTransactions:
         # funds += calculate_base_farmer_reward(0)
         await asyncio.sleep(2)
         print(await wallet.get_confirmed_balance(), funds)
-        await time_out_assert(10, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
 
     @pytest.mark.asyncio
     async def test_tx_propagation(self, three_nodes_two_wallets, self_hostname):
         num_blocks = 5
-        full_nodes, wallets = three_nodes_two_wallets
+        full_nodes, wallets, _ = three_nodes_two_wallets
 
         wallet_0, wallet_server_0 = wallets[0]
         wallet_1, wallet_server_1 = wallets[1]
@@ -68,7 +68,7 @@ class TestTransactions:
         funds = sum(
             [calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, num_blocks)]
         )
-        await time_out_assert(10, wallet_0.wallet_state_manager.main_wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet_0.wallet_state_manager.main_wallet.get_confirmed_balance, funds)
 
         async def peak_height(fna: FullNodeAPI):
             peak: Optional[BlockRecord] = fna.full_node.blockchain.get_peak()
@@ -77,8 +77,8 @@ class TestTransactions:
             peak_height = peak.height
             return peak_height
 
-        await time_out_assert(10, peak_height, num_blocks, full_node_api_1)
-        await time_out_assert(10, peak_height, num_blocks, full_node_api_2)
+        await time_out_assert(20, peak_height, num_blocks, full_node_api_1)
+        await time_out_assert(20, peak_height, num_blocks, full_node_api_2)
 
         tx = await wallet_0.wallet_state_manager.main_wallet.generate_signed_transaction(10, ph1, 0)
         await wallet_0.wallet_state_manager.main_wallet.push_transaction(tx)
@@ -117,12 +117,12 @@ class TestTransactions:
             wallet_0.wallet_state_manager.main_wallet.get_confirmed_balance,
             (funds - 10),
         )
-        await time_out_assert(15, wallet_1.wallet_state_manager.main_wallet.get_confirmed_balance, 10)
+        await time_out_assert(20, wallet_1.wallet_state_manager.main_wallet.get_confirmed_balance, 10)
 
     @pytest.mark.asyncio
     async def test_mempool_tx_sync(self, three_nodes_two_wallets, self_hostname):
         num_blocks = 5
-        full_nodes, wallets = three_nodes_two_wallets
+        full_nodes, wallets, _ = three_nodes_two_wallets
 
         wallet_0, wallet_server_0 = wallets[0]
         full_node_api_0 = full_nodes[0]
@@ -150,7 +150,7 @@ class TestTransactions:
         funds = sum(
             [calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, num_blocks)]
         )
-        await time_out_assert(10, wallet_0.wallet_state_manager.main_wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet_0.wallet_state_manager.main_wallet.get_confirmed_balance, funds)
 
         tx = await wallet_0.wallet_state_manager.main_wallet.generate_signed_transaction(10, token_bytes(), 0)
         await wallet_0.wallet_state_manager.main_wallet.push_transaction(tx)
