@@ -1,6 +1,7 @@
-from typing import List
 from pathlib import Path
-from tests.block_tools import get_plot_dir
+from typing import List
+
+from chia.simulator.block_tools import get_plot_dir
 
 
 def get_test_plots(sub_dir: str = "") -> List[Path]:
