@@ -5,10 +5,15 @@ from pathlib import Path
 
 from chia.consensus.constants import ConsensusConstants
 from chia.full_node.full_node_api import FullNodeAPI
+from chia.protocols.shared_protocol import Capability
+from chia.server.server import ChiaServer
+from chia.server.start_data_layer import create_data_layer_service
 from chia.server.start_service import Service
+from chia.simulator.block_tools import BlockTools, create_block_tools_async, test_constants
+from chia.simulator.full_node_simulator import FullNodeSimulator
 from chia.util.hash import std_hash
 from chia.util.ints import uint16, uint32
-from tests.block_tools import BlockTools, create_block_tools_async, test_constants
+from chia.wallet.wallet_node import WalletNode
 from tests.setup_services import (
     setup_daemon,
     setup_farmer,
@@ -20,9 +25,12 @@ from tests.setup_services import (
     setup_vdf_clients,
     setup_wallet_node,
 )
-from tests.time_out_assert import time_out_assert_custom_interval
+from chia.simulator.time_out_assert import time_out_assert_custom_interval
 from tests.util.keyring import TempKeyring
-from tests.util.socket import find_available_listen_port
+from chia.simulator.socket import find_available_listen_port
+
+
+SimulatorsAndWallets = Tuple[List[FullNodeSimulator], List[Tuple[WalletNode, ChiaServer]], BlockTools]
 
 
 def cleanup_keyring(keyring: TempKeyring):
@@ -45,18 +53,49 @@ async def _teardown_nodes(node_aiters: List) -> None:
             pass
 
 
+async def setup_data_layer(local_bt):
+    # db_path = local_bt.root_path / f"{db_name}"
+    # if db_path.exists():
+    #     db_path.unlink()
+    config = local_bt.config["data_layer"]
+    # config["database_path"] = db_name
+    # if introducer_port is not None:
+    #     config["introducer_peer"]["host"] = self_hostname
+    #     config["introducer_peer"]["port"] = introducer_port
+    # else:
+    #     config["introducer_peer"] = None
+    # config["dns_servers"] = []
+    # config["rpc_port"] = port + 1000
+    # overrides = config["network_overrides"]["constants"][config["selected_network"]]
+    # updated_constants = consensus_constants.replace_str_to_bytes(**overrides)
+    # if simulator:
+    #     kwargs = service_kwargs_for_full_node_simulator(local_bt.root_path, config, local_bt)
+    # else:
+    #     kwargs = service_kwargs_for_full_node(local_bt.root_path, config, updated_constants)
+
+    service = create_data_layer_service(local_bt.root_path, config, connect_to_daemon=False)
+
+    await service.start()
+
+    yield service._api
+
+    service.stop()
+    await service.wait_closed()
+
+
 async def setup_two_nodes(consensus_constants: ConsensusConstants, db_version: int, self_hostname: str):
     """
     Setup and teardown of two full nodes, with blockchains and separate DBs.
     """
 
     with TempKeyring(populate=True) as keychain1, TempKeyring(populate=True) as keychain2:
+        bt1 = await create_block_tools_async(constants=test_constants, keychain=keychain1)
         node_iters = [
             setup_full_node(
                 consensus_constants,
                 "blockchain_test.db",
                 self_hostname,
-                await create_block_tools_async(constants=test_constants, keychain=keychain1),
+                bt1,
                 simulator=False,
                 db_version=db_version,
             ),
@@ -73,7 +112,7 @@ async def setup_two_nodes(consensus_constants: ConsensusConstants, db_version: i
         fn1 = await node_iters[0].__anext__()
         fn2 = await node_iters[1].__anext__()
 
-        yield fn1, fn2, fn1.full_node.server, fn2.full_node.server
+        yield fn1, fn2, fn1.full_node.server, fn2.full_node.server, bt1
 
         await _teardown_nodes(node_iters)
 
@@ -110,7 +149,11 @@ async def setup_n_nodes(consensus_constants: ConsensusConstants, n: int, db_vers
 
 
 async def setup_node_and_wallet(
-    consensus_constants: ConsensusConstants, self_hostname: str, starting_height=None, key_seed=None, db_version=1
+    consensus_constants: ConsensusConstants,
+    self_hostname: str,
+    key_seed=None,
+    db_version=1,
+    disable_capabilities=None,
 ):
     with TempKeyring(populate=True) as keychain:
         btools = await create_block_tools_async(constants=test_constants, keychain=keychain)
@@ -122,13 +165,13 @@ async def setup_node_and_wallet(
                 btools,
                 simulator=False,
                 db_version=db_version,
+                disable_capabilities=disable_capabilities,
             ),
             setup_wallet_node(
                 btools.config["self_hostname"],
                 consensus_constants,
                 btools,
                 None,
-                starting_height=starting_height,
                 key_seed=key_seed,
             ),
         ]
@@ -136,7 +179,7 @@ async def setup_node_and_wallet(
         full_node_api = await node_iters[0].__anext__()
         wallet, s2 = await node_iters[1].__anext__()
 
-        yield full_node_api, wallet, full_node_api.full_node.server, s2
+        yield full_node_api, wallet, full_node_api.full_node.server, s2, btools
 
         await _teardown_nodes(node_iters)
 
@@ -145,31 +188,36 @@ async def setup_simulators_and_wallets(
     simulator_count: int,
     wallet_count: int,
     dic: Dict,
+    spam_filter_after_n_txs=200,
+    xch_spam_amount=1000000,
     *,
-    starting_height=None,
     key_seed=None,
     initial_num_public_keys=5,
     db_version=1,
     config_overrides: Optional[Dict] = None,
+    disable_capabilities: Optional[List[Capability]] = None,
 ):
     with TempKeyring(populate=True) as keychain1, TempKeyring(populate=True) as keychain2:
         simulators: List[FullNodeAPI] = []
         wallets = []
         node_iters = []
-
+        bt_tools: List[BlockTools] = []
         consensus_constants = constants_for_dic(dic)
         for index in range(0, simulator_count):
             db_name = f"blockchain_test_{index}_sim_and_wallets.db"
-            bt_tools = await create_block_tools_async(
-                consensus_constants, const_dict=dic, keychain=keychain1, config_overrides=config_overrides
+            bt_tools.append(
+                await create_block_tools_async(
+                    consensus_constants, const_dict=dic, keychain=keychain1, config_overrides=config_overrides
+                )
             )  # block tools modifies constants
             sim = setup_full_node(
-                bt_tools.constants,
-                bt_tools.config["self_hostname"],
+                bt_tools[index].constants,
+                bt_tools[index].config["self_hostname"],
                 db_name,
-                bt_tools,
+                bt_tools[index],
                 simulator=True,
                 db_version=db_version,
+                disable_capabilities=disable_capabilities,
             )
             simulators.append(await sim.__anext__())
             node_iters.append(sim)
@@ -179,72 +227,44 @@ async def setup_simulators_and_wallets(
                 seed = std_hash(uint32(index))
             else:
                 seed = key_seed
-            bt_tools = await create_block_tools_async(
-                consensus_constants, const_dict=dic, keychain=keychain2, config_overrides=config_overrides
-            )  # block tools modifies constants
+            if index > (len(bt_tools) - 1):
+                wallet_bt_tools = await create_block_tools_async(
+                    consensus_constants, const_dict=dic, keychain=keychain2, config_overrides=config_overrides
+                )  # block tools modifies constants
+            else:
+                wallet_bt_tools = bt_tools[index]
             wlt = setup_wallet_node(
-                bt_tools.config["self_hostname"],
-                bt_tools.constants,
-                bt_tools,
+                wallet_bt_tools.config["self_hostname"],
+                wallet_bt_tools.constants,
+                wallet_bt_tools,
+                spam_filter_after_n_txs,
+                xch_spam_amount,
                 None,
                 key_seed=seed,
-                starting_height=starting_height,
                 initial_num_public_keys=initial_num_public_keys,
             )
             wallets.append(await wlt.__anext__())
             node_iters.append(wlt)
 
-        yield simulators, wallets
+        yield simulators, wallets, bt_tools[0]
 
         await _teardown_nodes(node_iters)
 
 
-async def setup_harvester_farmer(
-    bt: BlockTools, tmp_path: Path, consensus_constants: ConsensusConstants, *, start_services: bool
-):
-    if start_services:
-        farmer_port = uint16(0)
-    else:
-        # If we don't start the services, we won't be able to get the farmer port, which the harvester needs
-        farmer_port = uint16(find_available_listen_port("farmer_server"))
-
-    farmer_setup_iter = setup_farmer(
-        bt,
-        tmp_path / "farmer",
-        bt.config["self_hostname"],
-        consensus_constants,
-        uint16(0),
-        start_service=start_services,
-        port=farmer_port,
-    )
-
-    farmer_service = await farmer_setup_iter.__anext__()
-    farmer_port = farmer_service._server._port
-    node_iters = [
-        setup_harvester(
-            bt,
-            tmp_path / "harvester",
-            bt.config["self_hostname"],
-            farmer_port,
-            consensus_constants,
-            start_services,
-        ),
-        farmer_setup_iter,
-    ]
-
-    harvester_service = await node_iters[0].__anext__()
-
-    yield harvester_service, farmer_service
-
-    await _teardown_nodes(node_iters)
-
-
 async def setup_farmer_multi_harvester(
     block_tools: BlockTools,
     harvester_count: int,
     temp_dir: Path,
     consensus_constants: ConsensusConstants,
-) -> AsyncIterator[Tuple[List[Service], Service]]:
+    *,
+    start_services: bool,
+) -> AsyncIterator[Tuple[List[Service], Service, BlockTools]]:
+
+    if start_services:
+        farmer_port = uint16(0)
+    else:
+        # If we don't start the services, we won't be able to get the farmer port, which the harvester needs
+        farmer_port = uint16(find_available_listen_port("farmer_server"))
 
     node_iterators = [
         setup_farmer(
@@ -252,7 +272,8 @@ async def setup_farmer_multi_harvester(
             temp_dir / "farmer",
             block_tools.config["self_hostname"],
             consensus_constants,
-            uint16(0),
+            port=farmer_port,
+            start_service=start_services,
         )
     ]
     farmer_service = await node_iterators[0].__anext__()
@@ -267,7 +288,7 @@ async def setup_farmer_multi_harvester(
                 block_tools.config["self_hostname"],
                 farmer_port,
                 consensus_constants,
-                False,
+                start_service=start_services,
             )
         )
 
@@ -276,7 +297,7 @@ async def setup_farmer_multi_harvester(
         harvester_service = await node.__anext__()
         harvester_services.append(harvester_service)
 
-    yield harvester_services, farmer_service
+    yield harvester_services, farmer_service, block_tools
 
     for harvester_service in harvester_services:
         harvester_service.stop()
