@@ -4,47 +4,67 @@ import signal
 import sqlite3
 from pathlib import Path
 from secrets import token_bytes
-from typing import AsyncGenerator, Optional
+from typing import AsyncGenerator, List, Optional, Tuple
 
 from chia.cmds.init_funcs import init
 from chia.consensus.constants import ConsensusConstants
-from chia.daemon.server import WebSocketServer, daemon_launch_lock_path, singleton
-from chia.server.start_farmer import service_kwargs_for_farmer
-from chia.server.start_full_node import service_kwargs_for_full_node
-from chia.server.start_harvester import service_kwargs_for_harvester
-from chia.server.start_introducer import service_kwargs_for_introducer
-from chia.server.start_service import Service
-from chia.server.start_timelord import service_kwargs_for_timelord
-from chia.server.start_wallet import service_kwargs_for_wallet
-from chia.simulator.start_simulator import service_kwargs_for_full_node_simulator
+from chia.daemon.server import WebSocketServer, daemon_launch_lock_path
+from chia.protocols.shared_protocol import Capability, capabilities
+from chia.server.start_farmer import create_farmer_service
+from chia.server.start_full_node import create_full_node_service
+from chia.server.start_harvester import create_harvester_service
+from chia.server.start_introducer import create_introducer_service
+from chia.server.start_timelord import create_timelord_service
+from chia.server.start_wallet import create_wallet_service
+from chia.simulator.block_tools import BlockTools
+from chia.simulator.start_simulator import create_full_node_simulator_service
 from chia.timelord.timelord_launcher import kill_processes, spawn_process
 from chia.util.bech32m import encode_puzzle_hash
 from chia.util.config import lock_and_load_config, save_config
 from chia.util.ints import uint16
 from chia.util.keychain import bytes_to_mnemonic
-from tests.block_tools import BlockTools
+from chia.util.lock import Lockfile
 from tests.util.keyring import TempKeyring
 
 log = logging.getLogger(__name__)
 
 
+def get_capabilities(disable_capabilities_values: Optional[List[Capability]]) -> List[Tuple[uint16, str]]:
+    if disable_capabilities_values is not None:
+        try:
+            if Capability.BASE in disable_capabilities_values:
+                # BASE capability cannot be removed
+                disable_capabilities_values.remove(Capability.BASE)
+
+            updated_capabilities = []
+            for capability in capabilities:
+                if Capability(int(capability[0])) in disable_capabilities_values:
+                    # "0" means capability is disabled
+                    updated_capabilities.append((capability[0], "0"))
+                else:
+                    updated_capabilities.append(capability)
+            return updated_capabilities
+        except Exception:
+            logging.getLogger(__name__).exception("Error disabling capabilities, defaulting to all capabilities")
+    return capabilities.copy()
+
+
 async def setup_daemon(btools: BlockTools) -> AsyncGenerator[WebSocketServer, None]:
     root_path = btools.root_path
     config = btools.config
     assert "daemon_port" in config
-    lockfile = singleton(daemon_launch_lock_path(root_path))
     crt_path = root_path / config["daemon_ssl"]["private_crt"]
     key_path = root_path / config["daemon_ssl"]["private_key"]
     ca_crt_path = root_path / config["private_ssl_ca"]["crt"]
     ca_key_path = root_path / config["private_ssl_ca"]["key"]
-    assert lockfile is not None
-    shutdown_event = asyncio.Event()
-    ws_server = WebSocketServer(root_path, ca_crt_path, ca_key_path, crt_path, key_path, shutdown_event)
-    await ws_server.start()
+    with Lockfile.create(daemon_launch_lock_path(root_path)):
+        shutdown_event = asyncio.Event()
+        ws_server = WebSocketServer(root_path, ca_crt_path, ca_key_path, crt_path, key_path, shutdown_event)
+        await ws_server.start()
 
-    yield ws_server
+        yield ws_server
 
-    await ws_server.stop()
+        await ws_server.stop()
 
 
 async def setup_full_node(
@@ -58,6 +78,7 @@ async def setup_full_node(
     sanitize_weight_proof_only=False,
     connect_to_daemon=False,
     db_version=1,
+    disable_capabilities: Optional[List[Capability]] = None,
 ):
     db_path = local_bt.root_path / f"{db_name}"
     if db_path.exists():
@@ -71,36 +92,43 @@ async def setup_full_node(
 
     if connect_to_daemon:
         assert local_bt.config["daemon_port"] is not None
-    config = local_bt.config["full_node"]
-
-    config["database_path"] = db_name
-    config["send_uncompact_interval"] = send_uncompact_interval
-    config["target_uncompact_proofs"] = 30
-    config["peer_connect_interval"] = 50
-    config["sanitize_weight_proof_only"] = sanitize_weight_proof_only
+    config = local_bt.config
+    service_config = config["full_node"]
+    service_config["database_path"] = db_name
+    service_config["send_uncompact_interval"] = send_uncompact_interval
+    service_config["target_uncompact_proofs"] = 30
+    service_config["peer_connect_interval"] = 50
+    service_config["sanitize_weight_proof_only"] = sanitize_weight_proof_only
     if introducer_port is not None:
-        config["introducer_peer"]["host"] = self_hostname
-        config["introducer_peer"]["port"] = introducer_port
+        service_config["introducer_peer"]["host"] = self_hostname
+        service_config["introducer_peer"]["port"] = introducer_port
     else:
-        config["introducer_peer"] = None
-    config["dns_servers"] = []
-    config["port"] = 0
-    config["rpc_port"] = 0
-    overrides = config["network_overrides"]["constants"][config["selected_network"]]
+        service_config["introducer_peer"] = None
+    service_config["dns_servers"] = []
+    service_config["port"] = 0
+    service_config["rpc_port"] = 0
+    config["simulator"]["auto_farm"] = False  # Disable Auto Farm for tests
+    config["simulator"]["use_current_time"] = False  # Disable Real timestamps when running tests
+    overrides = service_config["network_overrides"]["constants"][service_config["selected_network"]]
     updated_constants = consensus_constants.replace_str_to_bytes(**overrides)
+    local_bt.change_config(config)
+    override_capabilities = None if disable_capabilities is None else get_capabilities(disable_capabilities)
     if simulator:
-        kwargs = service_kwargs_for_full_node_simulator(local_bt.root_path, config, local_bt)
+        service = create_full_node_simulator_service(
+            local_bt.root_path,
+            config,
+            local_bt,
+            connect_to_daemon=connect_to_daemon,
+            override_capabilities=override_capabilities,
+        )
     else:
-        kwargs = service_kwargs_for_full_node(local_bt.root_path, config, updated_constants)
-
-    kwargs.update(
-        parse_cli_args=False,
-        connect_to_daemon=connect_to_daemon,
-        service_name_prefix="test_",
-    )
-
-    service = Service(**kwargs, running_new_process=False)
-
+        service = create_full_node_service(
+            local_bt.root_path,
+            config,
+            updated_constants,
+            connect_to_daemon=connect_to_daemon,
+            override_capabilities=override_capabilities,
+        )
     await service.start()
 
     yield service._api
@@ -117,19 +145,21 @@ async def setup_wallet_node(
     self_hostname: str,
     consensus_constants: ConsensusConstants,
     local_bt: BlockTools,
+    spam_filter_after_n_txs=200,
+    xch_spam_amount=1000000,
     full_node_port=None,
     introducer_port=None,
     key_seed=None,
-    starting_height=None,
     initial_num_public_keys=5,
 ):
     with TempKeyring(populate=True) as keychain:
-        config = local_bt.config["wallet"]
-        config["port"] = 0
-        config["rpc_port"] = 0
-        if starting_height is not None:
-            config["starting_height"] = starting_height
-        config["initial_num_public_keys"] = initial_num_public_keys
+        config = local_bt.config
+        service_config = config["wallet"]
+        service_config["port"] = 0
+        service_config["rpc_port"] = 0
+        service_config["initial_num_public_keys"] = initial_num_public_keys
+        service_config["spam_filter_after_n_txs"] = spam_filter_after_n_txs
+        service_config["xch_spam_amount"] = xch_spam_amount
 
         entropy = token_bytes(32)
         if key_seed is None:
@@ -144,32 +174,31 @@ async def setup_wallet_node(
 
         if db_path.exists():
             db_path.unlink()
-        config["database_path"] = str(db_name)
-        config["testing"] = True
+        service_config["database_path"] = str(db_name)
+        service_config["testing"] = True
 
-        config["introducer_peer"]["host"] = self_hostname
+        service_config["introducer_peer"]["host"] = self_hostname
         if introducer_port is not None:
-            config["introducer_peer"]["port"] = introducer_port
-            config["peer_connect_interval"] = 10
+            service_config["introducer_peer"]["port"] = introducer_port
+            service_config["peer_connect_interval"] = 10
         else:
-            config["introducer_peer"] = None
+            service_config["introducer_peer"] = None
 
         if full_node_port is not None:
-            config["full_node_peer"] = {}
-            config["full_node_peer"]["host"] = self_hostname
-            config["full_node_peer"]["port"] = full_node_port
+            service_config["full_node_peer"] = {}
+            service_config["full_node_peer"]["host"] = self_hostname
+            service_config["full_node_peer"]["port"] = full_node_port
         else:
-            del config["full_node_peer"]
+            del service_config["full_node_peer"]
 
-        kwargs = service_kwargs_for_wallet(local_bt.root_path, config, consensus_constants, keychain)
-        kwargs.update(
-            parse_cli_args=False,
+        service = create_wallet_service(
+            local_bt.root_path,
+            config,
+            consensus_constants,
+            keychain,
             connect_to_daemon=False,
-            service_name_prefix="test_",
         )
 
-        service = Service(**kwargs, running_new_process=False)
-
         await service.start()
 
         yield service._node, service._node.server
@@ -201,15 +230,13 @@ async def setup_harvester(
         config["harvester"]["farmer_peer"]["port"] = int(farmer_port)
         config["harvester"]["plot_directories"] = [str(b_tools.plot_dir.resolve())]
         save_config(root_path, "config.yaml", config)
-    kwargs = service_kwargs_for_harvester(root_path, config["harvester"], consensus_constants)
-    kwargs.update(
-        parse_cli_args=False,
+    service = create_harvester_service(
+        root_path,
+        config,
+        consensus_constants,
         connect_to_daemon=False,
-        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs, running_new_process=False)
-
     if start_service:
         await service.start()
 
@@ -235,30 +262,30 @@ async def setup_farmer(
         root_config["selected_network"] = "testnet0"
         root_config["farmer"]["selected_network"] = "testnet0"
         save_config(root_path, "config.yaml", root_config)
-    config = root_config["farmer"]
+    service_config = root_config["farmer"]
     config_pool = root_config["pool"]
 
-    config["xch_target_address"] = encode_puzzle_hash(b_tools.farmer_ph, "xch")
-    config["pool_public_keys"] = [bytes(pk).hex() for pk in b_tools.pool_pubkeys]
-    config["port"] = port
-    config["rpc_port"] = uint16(0)
+    service_config["xch_target_address"] = encode_puzzle_hash(b_tools.farmer_ph, "xch")
+    service_config["pool_public_keys"] = [bytes(pk).hex() for pk in b_tools.pool_pubkeys]
+    service_config["port"] = port
+    service_config["rpc_port"] = uint16(0)
     config_pool["xch_target_address"] = encode_puzzle_hash(b_tools.pool_ph, "xch")
 
     if full_node_port:
-        config["full_node_peer"]["host"] = self_hostname
-        config["full_node_peer"]["port"] = full_node_port
+        service_config["full_node_peer"]["host"] = self_hostname
+        service_config["full_node_peer"]["port"] = full_node_port
     else:
-        del config["full_node_peer"]
+        del service_config["full_node_peer"]
 
-    kwargs = service_kwargs_for_farmer(root_path, config, config_pool, consensus_constants, b_tools.local_keychain)
-    kwargs.update(
-        parse_cli_args=False,
+    service = create_farmer_service(
+        root_path,
+        root_config,
+        config_pool,
+        consensus_constants,
+        b_tools.local_keychain,
         connect_to_daemon=False,
-        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs, running_new_process=False)
-
     if start_service:
         await service.start()
 
@@ -269,19 +296,13 @@ async def setup_farmer(
 
 
 async def setup_introducer(bt: BlockTools, port):
-    kwargs = service_kwargs_for_introducer(
+    service = create_introducer_service(
         bt.root_path,
-        bt.config["introducer"],
-    )
-    kwargs.update(
+        bt.config,
         advertised_port=port,
-        parse_cli_args=False,
         connect_to_daemon=False,
-        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs, running_new_process=False)
-
     await service.start()
 
     yield service._api, service._node.server
@@ -291,32 +312,34 @@ async def setup_introducer(bt: BlockTools, port):
 
 
 async def setup_vdf_client(bt: BlockTools, self_hostname: str, port):
-    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1, bt.config.get("prefer_ipv6")))
+    lock = asyncio.Lock()
+    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1, lock, bt.config.get("prefer_ipv6")))
 
     def stop():
-        asyncio.create_task(kill_processes())
+        asyncio.create_task(kill_processes(lock))
 
     asyncio.get_running_loop().add_signal_handler(signal.SIGTERM, stop)
     asyncio.get_running_loop().add_signal_handler(signal.SIGINT, stop)
 
     yield vdf_task_1
-    await kill_processes()
+    await kill_processes(lock)
 
 
 async def setup_vdf_clients(bt: BlockTools, self_hostname: str, port):
-    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1, bt.config.get("prefer_ipv6")))
-    vdf_task_2 = asyncio.create_task(spawn_process(self_hostname, port, 2, bt.config.get("prefer_ipv6")))
-    vdf_task_3 = asyncio.create_task(spawn_process(self_hostname, port, 3, bt.config.get("prefer_ipv6")))
+    lock = asyncio.Lock()
+    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1, lock, bt.config.get("prefer_ipv6")))
+    vdf_task_2 = asyncio.create_task(spawn_process(self_hostname, port, 2, lock, bt.config.get("prefer_ipv6")))
+    vdf_task_3 = asyncio.create_task(spawn_process(self_hostname, port, 3, lock, bt.config.get("prefer_ipv6")))
 
     def stop():
-        asyncio.create_task(kill_processes())
+        asyncio.create_task(kill_processes(lock))
 
     asyncio.get_running_loop().add_signal_handler(signal.SIGTERM, stop)
     asyncio.get_running_loop().add_signal_handler(signal.SIGINT, stop)
 
     yield vdf_task_1, vdf_task_2, vdf_task_3
 
-    await kill_processes()
+    await kill_processes(lock)
 
 
 async def setup_timelord(
@@ -326,23 +349,22 @@ async def setup_timelord(
     b_tools: BlockTools,
     vdf_port: uint16 = uint16(0),
 ):
-    config = b_tools.config["timelord"]
-    config["full_node_peer"]["port"] = full_node_port
-    config["bluebox_mode"] = sanitizer
-    config["fast_algorithm"] = False
-    config["vdf_server"]["port"] = vdf_port
-    config["start_rpc_server"] = True
-    config["rpc_port"] = uint16(0)
+    config = b_tools.config
+    service_config = config["timelord"]
+    service_config["full_node_peer"]["port"] = full_node_port
+    service_config["bluebox_mode"] = sanitizer
+    service_config["fast_algorithm"] = False
+    service_config["vdf_server"]["port"] = vdf_port
+    service_config["start_rpc_server"] = True
+    service_config["rpc_port"] = uint16(0)
 
-    kwargs = service_kwargs_for_timelord(b_tools.root_path, config, consensus_constants)
-    kwargs.update(
-        parse_cli_args=False,
+    service = create_timelord_service(
+        b_tools.root_path,
+        config,
+        consensus_constants,
         connect_to_daemon=False,
-        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs, running_new_process=False)
-
     await service.start()
 
     yield service._api, service._node.server
