@@ -14,8 +14,7 @@ from chia.full_node.hint_store import HintStore
 from chia.types.full_block import FullBlock
 from chia.util.db_wrapper import DBWrapper2
 from chia.util.default_root import DEFAULT_ROOT_PATH
-from chia.util.path import mkdir
-from tests.block_tools import BlockTools
+from chia.simulator.block_tools import BlockTools
 
 
 async def create_blockchain(constants: ConsensusConstants, db_version: int):
@@ -59,7 +58,7 @@ def persistent_blocks(
     if ci is not None and not file_path.exists():
         raise Exception(f"Running in CI and expected path not found: {file_path!r}")
 
-    mkdir(block_path_dir)
+    block_path_dir.mkdir(parents=True, exist_ok=True)
 
     if file_path.exists():
         print(f"File found at: {file_path}")
