@@ -1,4 +1,8 @@
-async def validate_get_routes(client, api):
+from chia.rpc.rpc_client import RpcClient
+from chia.rpc.rpc_server import RpcApiProtocol
+
+
+async def validate_get_routes(client: RpcClient, api: RpcApiProtocol):
     routes_client = (await client.fetch("get_routes", {}))["routes"]
     assert len(routes_client) > 0
     routes_api = list(api.get_routes().keys())
