@@ -1,7 +1,8 @@
 import random
-from typing import Iterator
+from typing import Generator, Iterator, List, Optional
 
 import pytest
+from blspy import G1Element, G2Element
 
 from benchmarks.utils import rand_bytes, rand_g1, rand_g2, rand_hash, rand_vdf, rand_vdf_proof, rewards
 from chia.types.blockchain_format.foliage import Foliage, FoliageBlockData, FoliageTransactionBlock, TransactionsInfo
@@ -9,45 +10,49 @@ from chia.types.blockchain_format.pool_target import PoolTarget
 from chia.types.blockchain_format.program import SerializedProgram
 from chia.types.blockchain_format.proof_of_space import ProofOfSpace
 from chia.types.blockchain_format.reward_chain_block import RewardChainBlock
+from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.blockchain_format.slots import (
     ChallengeChainSubSlot,
     InfusedChallengeChainSubSlot,
     RewardChainSubSlot,
     SubSlotProofs,
 )
+from chia.types.blockchain_format.vdf import VDFInfo, VDFProof
 from chia.types.end_of_slot_bundle import EndOfSubSlotBundle
 from chia.types.full_block import FullBlock
-from chia.util.full_block_utils import generator_from_block
+from chia.types.header_block import HeaderBlock
+from chia.util.full_block_utils import generator_from_block, header_block_from_block
+from chia.util.generator_tools import get_block_header
 from chia.util.ints import uint8, uint32, uint64, uint128
 
-test_g2s = [rand_g2() for _ in range(10)]
-test_g1s = [rand_g1() for _ in range(10)]
-test_hashes = [rand_hash() for _ in range(100)]
-test_vdfs = [rand_vdf() for _ in range(100)]
-test_vdf_proofs = [rand_vdf_proof() for _ in range(100)]
+test_g2s: List[G2Element] = [rand_g2() for _ in range(10)]
+test_g1s: List[G1Element] = [rand_g1() for _ in range(10)]
+test_hashes: List[bytes32] = [rand_hash() for _ in range(100)]
+test_vdfs: List[VDFInfo] = [rand_vdf() for _ in range(100)]
+test_vdf_proofs: List[VDFProof] = [rand_vdf_proof() for _ in range(100)]
 
 
-def g2():
+def g2() -> G2Element:
     return random.sample(test_g2s, 1)[0]
 
 
-def g1():
+def g1() -> G1Element:
     return random.sample(test_g1s, 1)[0]
 
 
-def hsh():
+def hsh() -> bytes32:
     return random.sample(test_hashes, 1)[0]
 
 
-def vdf():
+def vdf() -> VDFInfo:
     return random.sample(test_vdfs, 1)[0]
 
 
-def vdf_proof():
+def vdf_proof() -> VDFProof:
     return random.sample(test_vdf_proofs, 1)[0]
 
 
-def get_proof_of_space():
+def get_proof_of_space() -> Generator[ProofOfSpace, None, None]:
     for pool_pk in [g1(), None]:
         for plot_hash in [hsh(), None]:
             yield ProofOfSpace(
@@ -60,7 +65,7 @@ def get_proof_of_space():
             )
 
 
-def get_reward_chain_block(height):
+def get_reward_chain_block(height: uint32) -> Generator[RewardChainBlock, None, None]:
     for has_transactions in [True, False]:
         for challenge_chain_sp_vdf in [vdf(), None]:
             for reward_chain_sp_vdf in [vdf(), None]:
@@ -87,7 +92,7 @@ def get_reward_chain_block(height):
                         )
 
 
-def get_foliage_block_data():
+def get_foliage_block_data() -> Generator[FoliageBlockData, None, None]:
     for pool_signature in [g2(), None]:
         pool_target = PoolTarget(
             hsh(),  # puzzle_hash
@@ -103,7 +108,7 @@ def get_foliage_block_data():
         )
 
 
-def get_foliage():
+def get_foliage() -> Generator[Foliage, None, None]:
     for foliage_block_data in get_foliage_block_data():
         for foliage_transaction_block_hash in [hsh(), None]:
             for foliage_transaction_block_signature in [g2(), None]:
@@ -117,7 +122,7 @@ def get_foliage():
                 )
 
 
-def get_foliage_transaction_block():
+def get_foliage_transaction_block() -> Generator[Optional[FoliageTransactionBlock], None, None]:
     yield None
     timestamp = uint64(1631794488)
     yield FoliageTransactionBlock(
@@ -130,23 +135,24 @@ def get_foliage_transaction_block():
     )
 
 
-def get_transactions_info(height):
-    yield None
-    farmer_coin, pool_coin = rewards(uint32(height))
-    reward_claims_incorporated = [farmer_coin, pool_coin]
-    fees = uint64(random.randint(0, 150000))
-
-    yield TransactionsInfo(
-        hsh(),  # generator_root
-        hsh(),  # generator_refs_root
-        g2(),  # aggregated_signature
-        fees,
-        uint64(random.randint(0, 12000000000)),  # cost
-        reward_claims_incorporated,
-    )
+def get_transactions_info(height: uint32, foliage_transaction_block: Optional[FoliageTransactionBlock]):
+    if not foliage_transaction_block:
+        yield None
+    else:
+        farmer_coin, pool_coin = rewards(uint32(height))
+        reward_claims_incorporated = [farmer_coin, pool_coin]
+        fees = uint64(random.randint(0, 150000))
+        yield TransactionsInfo(
+            hsh(),  # generator_root
+            hsh(),  # generator_refs_root
+            g2(),  # aggregated_signature
+            fees,
+            uint64(random.randint(0, 12000000000)),  # cost
+            reward_claims_incorporated,
+        )
 
 
-def get_challenge_chain_sub_slot():
+def get_challenge_chain_sub_slot() -> Generator[ChallengeChainSubSlot, None, None]:
     for infused_chain_sub_slot_hash in [hsh(), None]:
         for sub_epoch_summary_hash in [hsh(), None]:
             for new_sub_slot_iters in [uint64(random.randint(0, 4000000000)), None]:
@@ -160,7 +166,7 @@ def get_challenge_chain_sub_slot():
                     )
 
 
-def get_reward_chain_sub_slot():
+def get_reward_chain_sub_slot() -> Generator[RewardChainSubSlot, None, None]:
     for infused_challenge_chain_sub_slot_hash in [hsh(), None]:
         yield RewardChainSubSlot(
             vdf(),  # end_of_slot_vdf
@@ -170,7 +176,7 @@ def get_reward_chain_sub_slot():
         )
 
 
-def get_sub_slot_proofs():
+def get_sub_slot_proofs() -> Generator[SubSlotProofs, None, None]:
     for infused_challenge_chain_slot_proof in [vdf_proof(), None]:
         yield SubSlotProofs(
             vdf_proof(),  # challenge_chain_slot_proof
@@ -179,7 +185,7 @@ def get_sub_slot_proofs():
         )
 
 
-def get_end_of_sub_slot():
+def get_end_of_sub_slot() -> Generator[EndOfSubSlotBundle, None, None]:
     for challenge_chain in get_challenge_chain_sub_slot():
         for infused_challenge_chain in [InfusedChallengeChainSubSlot(vdf()), None]:
             for reward_chain in get_reward_chain_sub_slot():
@@ -192,27 +198,25 @@ def get_end_of_sub_slot():
                     )
 
 
-def get_finished_sub_slots():
+def get_finished_sub_slots() -> Generator[List[EndOfSubSlotBundle], None, None]:
     yield []
     yield [s for s in get_end_of_sub_slot()]
 
 
 def get_full_blocks() -> Iterator[FullBlock]:
-
     random.seed(123456789)
 
     generator = SerializedProgram.from_bytes(bytes.fromhex("ff01820539"))
 
     for foliage in get_foliage():
         for foliage_transaction_block in get_foliage_transaction_block():
-            height = random.randint(0, 1000000)
+            height = uint32(random.randint(0, 1000000))
             for reward_chain_block in get_reward_chain_block(height):
-                for transactions_info in get_transactions_info(height):
+                for transactions_info in get_transactions_info(height, foliage_transaction_block):
                     for challenge_chain_sp_proof in [vdf_proof(), None]:
                         for reward_chain_sp_proof in [vdf_proof(), None]:
                             for infused_challenge_chain_ip_proof in [vdf_proof(), None]:
                                 for finished_sub_slots in get_finished_sub_slots():
-
                                     yield FullBlock(
                                         finished_sub_slots,
                                         reward_chain_block,
@@ -238,9 +242,15 @@ class TestFullBlockParser:
         # generator_from_block() successfully parses out the generator object
         # correctly
         for block in get_full_blocks():
-
             block_bytes = bytes(block)
             gen = generator_from_block(block_bytes)
             assert gen == block.transactions_generator
             # this doubles the run-time of this test, with questionable utility
             # assert gen == FullBlock.from_bytes(block_bytes).transactions_generator
+
+    @pytest.mark.asyncio
+    async def test_header_block(self):
+        for block in get_full_blocks():
+            hb: HeaderBlock = get_block_header(block, [], [])
+            hb_bytes = header_block_from_block(memoryview(bytes(block)))
+            assert HeaderBlock.from_bytes(hb_bytes) == hb
